@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2002, 2005, 2007-2012, 2014-2016 Free Software
+   Copyright (C) 1999-2002, 2005, 2007-2012, 2014-2017 Free Software
    Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
@@ -18,6 +18,7 @@
 #include "comsat.h"
 #include "mailutils/syslog.h"
 #include "mailutils/cli.h"
+#include "mailutils/sockaddr.h"
 
 #ifndef PATH_DEV
 # define PATH_DEV "/dev"
@@ -194,11 +195,11 @@ struct mu_cfg_param comsat_cfg_param[] = {
 static char const *alt_args[] = { N_("--test MBOX-URL MSG-QID"), NULL };
 
 static struct mu_cli_setup cli = {
-  options,
-  comsat_cfg_param,
-  N_("GNU comsatd -- notify users about incoming mail"),
-  "",
-  alt_args,
+  .optv = options,
+  .cfg =comsat_cfg_param,
+  .prog_doc = N_("GNU comsatd -- notify users about incoming mail"),
+  .prog_alt_args = alt_args,
+  .server = 1
 };
 
 static char *capa[] = {
@@ -371,7 +372,7 @@ comsat_connection (int fd, struct sockaddr *sa, int salen,
     return 0;
   if (pconf->transcript)
     {
-      char *p = mu_sockaddr_to_astr (sa, salen);
+      char *p = mu_sys_sockaddr_to_astr (sa, salen);
       mu_diag_output (MU_DIAG_INFO,
 		      ngettext ("received %lu byte from %s",
 				"received %lu bytes from %s",
@@ -568,7 +569,7 @@ main (int argc, char **argv)
 
   save_argv = argv;
   
-  mu_cli (argc, argv, &cli, capa, NULL, &argc, &argv);
+  mu_cli (argc, argv, &cli, capa, server, &argc, &argv);
 
   if (test_mode)
     {
