@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2004, 2006-2007, 2009-2012 Free Software Foundation,
-   Inc.
+   Copyright (C) 2004, 2006-2007, 2009-2012, 2014-2017 Free Software
+   Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
