@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001-2012, 2014-2015 Free Software Foundation,
+   Copyright (C) 1999, 2001-2012, 2014-2017 Free Software Foundation,
    Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
@@ -120,9 +120,9 @@ directory_size (const char *dirname, mu_off_t *size)
 mu_off_t used_size;
 
 void
-quota_setup ()
+quota_setup (void)
 {
-  directory_size (imap4d_homedir, &used_size);
+  directory_size (real_homedir, &used_size);
 }
 
 int
@@ -132,19 +132,17 @@ quota_check (mu_off_t size)
   mu_mailbox_t mbox;
   mu_off_t total;
   int rc;
+  mu_record_t record;
   
   if (auth_data->quota == 0)
     return RESP_OK;
 
   total = used_size;
 
-  mailbox_name = namespace_getfullpath ("INBOX", NULL);
-  rc = mu_mailbox_create (&mbox, mailbox_name);
+  mailbox_name = namespace_get_name ("INBOX", &record, NULL);
+  rc = mu_mailbox_create_from_record (&mbox, record, mailbox_name);
   if (rc)
-    {
-      mu_diag_funcall (MU_DIAG_ERROR, "mu_mailbox_create", mailbox_name, rc);
-      free (mailbox_name);
-    }
+    mu_diag_funcall (MU_DIAG_ERROR, "mu_mailbox_create", mailbox_name, rc);
   else
     {
       do
