@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2001, 2005, 2007, 2010-2012, 2014-2015 Free
+   Copyright (C) 1999-2001, 2005, 2007, 2010-2012, 2014-2017 Free
    Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
@@ -24,13 +24,14 @@
 #include <mailutils/types.h>
 #include <mailutils/log.h>
 #include <mailutils/debug.h>
+#include <mailutils/locus.h>
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-extern const char *mu_program_name;
-extern const char *mu_full_program_name;
+extern char *mu_program_name;
+extern char *mu_full_program_name;
 
 #define MU_DIAG_EMERG    MU_LOG_EMERG
 #define MU_DIAG_ALERT    MU_LOG_ALERT
@@ -51,8 +52,12 @@ void mu_diag_cont_printf (const char *fmt, ...) MU_PRINTFLIKE(1,2);
   
 void mu_diag_voutput (int, const char *, va_list);
 void mu_diag_output (int, const char *, ...) MU_PRINTFLIKE(2,3);
-void mu_diag_at_locus (int level, struct mu_locus const *loc,
-		       const char *fmt, ...);
+void mu_vdiag_at_locus_range (int level, struct mu_locus_range const *loc,
+			      const char *fmt, va_list ap);
+void mu_diag_at_locus_point (int level, struct mu_locus_point const *loc,
+			     const char *fmt, ...);
+void mu_diag_at_locus_range (int level, struct mu_locus_range const *loc,
+			     const char *fmt, ...);
 
 int mu_diag_level_to_syslog (int level);
 const char *mu_diag_level_to_string (int level);
