@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2000, 2004-2005, 2007, 2010-2012, 2014-2015 Free
+   Copyright (C) 1999-2000, 2004-2005, 2007, 2010-2012, 2014-2017 Free
    Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
@@ -35,7 +35,8 @@ enum mu_itrctl_req
     mu_itrctl_insert,        /* Insert new element in the current position */
     mu_itrctl_insert_list,   /* Insert a list of elements */
     mu_itrctl_qry_direction, /* Query iteration direction */
-    mu_itrctl_set_direction  /* Set iteration direction */
+    mu_itrctl_set_direction, /* Set iteration direction */
+    mu_itrctl_count          /* Get number of elements */
   };
 
 extern int mu_iterator_create   (mu_iterator_t *, void *);
