@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2009-2012, 2014-2015 Free Software Foundation, Inc.
+   Copyright (C) 2009-2012, 2014-2017 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
 #include <string.h>
 
 static int
-python_init (const char *prog, mu_script_descr_t *pdescr)
+python_init (const char *prog, const char **env, mu_script_descr_t *pdescr)
 {
   *pdescr = (mu_script_descr_t) strdup (prog);
   if (!*pdescr)
