@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2002, 2005, 2007, 2009-2012, 2014-2015 Free
+   Copyright (C) 1999-2002, 2005, 2007, 2009-2012, 2014-2017 Free
    Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
@@ -39,6 +39,8 @@ struct mu_script_fun *script_tab[] = {
 int
 mu_script_debug_flags (const char *arg, char **endp)
 {
+  mu_debug_level_t lev;
+    
   for (; *arg; arg++)
     {
       switch (*arg)
@@ -48,11 +50,15 @@ mu_script_debug_flags (const char *arg, char **endp)
 	  break;
 
 	case 't':
-	  mu_script_debug_sieve |= MU_SIEVE_DEBUG_TRACE;
+	  mu_debug_get_category_level (mu_sieve_debug_handle, &lev);
+	  mu_debug_set_category_level (mu_sieve_debug_handle,
+			       lev | MU_DEBUG_LEVEL_MASK(MU_DEBUG_TRACE4));
 	  break;
 	  
 	case 'i':
-	  mu_script_debug_sieve |= MU_SIEVE_DEBUG_INSTR;
+	  mu_debug_get_category_level (mu_sieve_debug_handle, &lev);
+	  mu_debug_set_category_level (mu_sieve_debug_handle,
+			       lev | MU_DEBUG_LEVEL_MASK(MU_DEBUG_TRACE9));
 	  break;
 	  
 	case 'l':
@@ -102,9 +108,10 @@ mu_script_suffix_handler (const char *name)
 }
 
 int
-mu_script_init (mu_script_t scr, const char *name, mu_script_descr_t *pdescr)
+mu_script_init (mu_script_t scr, const char *name, const char **env,
+		mu_script_descr_t *pdescr)
 {
-  return scr->script_init ? scr->script_init (name, pdescr) : 0;
+  return scr->script_init ? scr->script_init (name, env, pdescr) : 0;
 }
 
 int
