@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2012, 2014-2016 Free Software Foundation, Inc.
+   Copyright (C) 1999-2012, 2014-2017 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -216,34 +216,6 @@ _amd_prop_store_off (struct _amd_data *amd, const char *name, mu_off_t val)
   return mu_property_set_value (amd->prop, name, p, 1);
 }
 
-/* Backward-compatible size file support */
-static int
-read_size_file (struct _amd_data *amd)
-{
-  FILE *fp;
-  int rc;
-  char *name = mu_make_file_name (amd->name, _MU_AMD_SIZE_FILE_NAME);
-  if (!name)
-    return 1;
-  fp = fopen (name, "r");
-  if (fp)
-    {
-      unsigned long size;
-      if (fscanf (fp, "%lu", &size) == 1)
-	{
-	  rc = _amd_prop_store_off (amd, _MU_AMD_PROP_SIZE, size);
-	}
-      else
-	rc = 1;
-      fclose (fp);
-      unlink (name);
-    }
-  else
-    rc = 1;
-  free (name);
-  return rc;
-}
-
 static int
 _amd_prop_create (struct _amd_data *amd)
 {
@@ -267,8 +239,7 @@ _amd_prop_create (struct _amd_data *amd)
       free (mhprop->filename);
       free (mhprop);
     }
-  else
-    read_size_file (amd);
+  
   return rc;
 }
 
@@ -471,6 +442,8 @@ amd_destroy (mu_mailbox_t mailbox)
   for (i = 0; i < amd->msg_count; i++)
     {
       mu_message_destroy (&amd->msg_array[i]->message, amd->msg_array[i]);
+      if (amd->msg_free)
+	amd->msg_free (amd->msg_array[i]);
       free (amd->msg_array[i]);
     }
   free (amd->msg_array);
@@ -543,11 +516,15 @@ amd_close (mu_mailbox_t mailbox)
   for (i = 0; i < amd->msg_count; i++)
     {
       mu_message_destroy (&amd->msg_array[i]->message, amd->msg_array[i]);
+      if (amd->msg_free)
+	amd->msg_free (amd->msg_array[i]);
       free (amd->msg_array[i]);
     }
   free (amd->msg_array);
   amd->msg_array = NULL;
 
+  mu_property_save (amd->prop);
+    
   amd->msg_count = 0; /* number of messages in the list */
   amd->msg_max = 0;   /* maximum message buffer capacity */
 
@@ -704,7 +681,7 @@ _amd_scan0 (struct _amd_data *amd, size_t msgno, size_t *pcount,
       _amd_prop_fetch_ulong (amd, _MU_AMD_PROP_UIDVALIDITY, &uidval) ||
       !uidval)
     {
-      uidval = (unsigned long)time (NULL);
+      uidval = (unsigned long) amd->mtime;
       _amd_prop_store_off (amd, _MU_AMD_PROP_UIDVALIDITY, uidval);
     }
   return 0;
@@ -1246,20 +1223,11 @@ amd_remove_mbox (mu_mailbox_t mailbox)
     {
       char *name;
 
-      name = mu_make_file_name (amd->name, _MU_AMD_SIZE_FILE_NAME);
+      name = mu_make_file_name (amd->name, _MU_AMD_PROP_FILE_NAME);
       if (!name)
 	return ENOMEM;
       if (unlink (name) && errno != ENOENT)
 	rc = errno;
-      else
-	{
-	  free (name);
-	  name = mu_make_file_name (amd->name, _MU_AMD_PROP_FILE_NAME);
-	  if (!name)
-	    return ENOMEM;
-	  if (unlink (name) && errno != ENOENT)
-	    rc = errno;
-	}
       free (name);
     }
 
@@ -2246,6 +2214,7 @@ amd_envelope_sender (mu_envelope_t envelope, char *buf, size_t len, size_t *psiz
 
   if (psize)
     *psize = len;
+  free (from);
   return 0;
 }
 
