@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003, 2009-2012, 2014-2015 Free Software Foundation,
+   Copyright (C) 2003, 2009-2012, 2014-2017 Free Software Foundation,
    Inc.
 
    This library is free software; you can redistribute it and/or
@@ -235,7 +235,7 @@ mu_charset_lookup (char *lang, char *terr)
     if (mu_c_strcasecmp (p->lang, lang) == 0
 	&& (terr == NULL 
 	    || p->terr == NULL
-	    || !mu_c_strcasecmp (p->terr, terr) == 0))
+	    || mu_c_strcasecmp (p->terr, terr) == 0))
       return p->charset;
   return NULL;
 }
