@@ -38,6 +38,8 @@
 #include <mailutils/header.h>
 #include <mailutils/errno.h>
 #include <mailutils/util.h>
+#include <mailutils/assoc.h>
+#include <mailutils/io.h>
 
 #include <mailutils/sys/mime.h>
 #include <mailutils/sys/message.h>
@@ -444,6 +446,13 @@ _mimepart_body_lines (mu_body_t body, size_t *plines)
 }
 
 /*------ Mime message/header functions for CREATING multipart message -----*/
+static int
+retain_charset (char const *name, void *value MU_ARG_UNUSED,
+		void *data MU_ARG_UNUSED)
+{
+  return strcmp (name, "charset") != 0;
+}
+		 
 static int
 _mime_set_content_type (mu_mime_t mime)
 {
@@ -465,7 +474,48 @@ _mime_set_content_type (mu_mime_t mime)
       if (mime->flags & MU_MIME_MULTIPART_MIXED)
 	content_type = "multipart/mixed; boundary=";
       else
-	content_type = "multipart/alternative; boundary=";
+	{
+	  size_t i;
+
+	  /* Make sure content disposition is not set for alternative
+	     parts */
+	  for (i = 0; i < mime->nmtp_parts; i++)
+	    {
+	      mu_header_t hdr;
+	      char *val;
+	      int rc;
+	      
+	      mu_message_get_header (mime->mtp_parts[i]->msg, &hdr);
+	      mu_header_remove (hdr, MU_HEADER_CONTENT_DISPOSITION, 1);
+
+	      rc = mu_header_aget_value_unfold (hdr, MU_HEADER_CONTENT_TYPE,
+						&val);
+	      if (rc == 0)
+		{
+		  mu_content_type_t ct;
+		  rc = mu_content_type_parse (val, NULL, &ct);
+		  if (rc == 0)
+		    {
+		      char *type;
+		      
+		      mu_assoc_mark (ct->param, retain_charset, NULL);
+		      mu_assoc_sweep (ct->param);
+
+		      rc = mu_asprintf (&type, "%s/%s", ct->type, ct->subtype);
+		      if (rc == 0)
+			{
+			  mu_mime_header_set (hdr,
+					      MU_HEADER_CONTENT_TYPE, type,
+					      ct->param);
+			  free (type);
+			}
+		      mu_content_type_destroy (&ct);
+		    }
+		}
+	    }
+	  
+	  content_type = "multipart/alternative; boundary=";
+	}
       if (mime->boundary == NULL)
 	{
 	  char boundary[128];
@@ -879,6 +929,9 @@ _mime_body_lines (mu_body_t body, size_t *plines)
   return 0;
 }
 
+#define MIME_MULTIPART_FLAGS \
+  (MU_MIME_MULTIPART_MIXED|MU_MIME_MULTIPART_ALT)
+
 int
 mu_mime_create (mu_mime_t *pmime, mu_message_t msg, int flags)
 {
@@ -886,9 +939,19 @@ mu_mime_create (mu_mime_t *pmime, mu_message_t msg, int flags)
   int ret = 0;
   size_t size;
   mu_body_t body;
-
+  
   if (pmime == NULL)
     return EINVAL;
+  
+  switch (flags & MIME_MULTIPART_FLAGS)
+    {
+    case MIME_MULTIPART_FLAGS:
+      return EINVAL;
+
+    case 0:
+      flags |= MU_MIME_MULTIPART_MIXED;
+    }
+  
   *pmime = NULL;
   if ((mime = calloc (1, sizeof (*mime))) == NULL)
     return ENOMEM;
@@ -931,7 +994,7 @@ mu_mime_create (mu_mime_t *pmime, mu_message_t msg, int flags)
     }
   else
     {
-      mime->flags |= MIME_NEW_MESSAGE | MU_MIME_MULTIPART_MIXED;
+      mime->flags |= MIME_NEW_MESSAGE;
     }
   if (ret != 0)
     {
@@ -1105,7 +1168,7 @@ mu_mime_get_message (mu_mime_t mime, mu_message_t *msg)
 	    {
 	      mu_message_set_header (mime->msg, mime->hdrs, mime);
 	      mu_header_set_value (mime->hdrs, MU_HEADER_MIME_VERSION, "1.0",
-				   0);
+				   1);
 	      if ((ret = _mime_set_content_type (mime)) == 0)
 		{
 		  if ((ret = mu_body_create (&body, mime->msg)) == 0)
