@@ -41,8 +41,6 @@ struct _mu_connection
 
 #define MU_SERVER_TIMEOUT 0x1
 
-unsigned long mu_session_id;
-
 struct _mu_server
 {
   int nfd;
@@ -111,7 +109,6 @@ connection_loop (mu_server_t srv, fd_set *fdset)
 	{
 	  int rc;
 
-	  ++mu_session_id;
 	  rc = conn->f_loop (conn->fd, conn->data, srv->server_data);
 	  switch (rc)
 	    {
@@ -300,11 +297,4 @@ mu_server_add_connection (mu_server_t srv,
   return 0;
 }
 
-int
-mu_acl_set_session_id (mu_acl_t acl)
-{
-  char sessidstr[9];
-  snprintf (sessidstr, sizeof sessidstr, "%08lx", mu_session_id);
-  return mu_acl_setenv (acl, "sessid", sessidstr);
-}
 
