@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2001, 2004-2005, 2007, 2010-2012, 2014-2015 Free
+   Copyright (C) 1999-2001, 2004-2005, 2007, 2010-2012, 2014-2017 Free
    Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
@@ -63,13 +63,13 @@ parse (const char *str)
 
       if (isgroup)
         {
-          mu_address_sget_personal (address, i, &buf);
-          printf ("group <%s>:\n", buf);
+          if (mu_address_sget_personal (address, i, &buf) == 0 && buf)
+	    printf ("group <%s>:\n", buf);
         }
       else
         {
-          mu_address_sget_email (address, i, &buf);
-          printf ("email <%s>:\n", buf);
+          if (mu_address_sget_email (address, i, &buf) == 0 && buf)
+	    printf ("email <%s>:\n", buf);
         }
 
       if (mu_address_sget_personal (address, i, &buf) == 0 && buf && !isgroup)
