@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2005, 2007-2012, 2014-2016 Free Software Foundation,
+   Copyright (C) 2005, 2007-2012, 2014-2017 Free Software Foundation,
    Inc.
 
    This library is free software; you can redistribute it and/or
@@ -60,24 +60,22 @@ const char *default_field_map =
 "shell=loginShell";
 
 static struct mu_ldap_module_config ldap_param;
-
-
 
-static int
-_cb2_field_map (const char *arg, void *data)
-{
-  int err;
-  int rc = mutil_parse_field_map (arg, &ldap_param.field_map, &err);
-  if (rc)
-    /* FIXME: this message can be misleading */
-    mu_error (_("error near element %d: %s"), err, mu_strerror (rc));
-  return 0;
-}
-
 static int
 cb_field_map (void *data, mu_config_value_t *val)
 {
-  return mu_cfg_string_value_cb (val, _cb2_field_map, NULL);
+  char *err_term;
+  int rc = mu_cfg_field_map (val, &ldap_param.field_map, &err_term);
+
+  if (rc)
+    {
+      if (err_term)
+	mu_error (_("error near %s: %s"), err_term, mu_strerror (rc));
+      else
+	mu_error ("%s", mu_strerror (rc));
+    }
+
+  return rc;
 }
 
 static struct mu_cfg_param mu_ldap_param[] = {
@@ -140,14 +138,18 @@ module_init (void *ptr)
   if (ldap_param.enable)
     {
       if (!ldap_param.getpwnam_filter)
-	ldap_param.getpwnam_filter = "(&(objectClass=posixAccount) (uid=%u))";
+	ldap_param.getpwnam_filter =
+	  "(&(objectClass=posixAccount) (uid=$user))";
       if (!ldap_param.getpwuid_filter)
 	ldap_param.getpwuid_filter =
-	  "&(objectClass=posixAccount) (uidNumber=%u))";
+	  "(&(objectClass=posixAccount) (uidNumber=$user))";
       if (!ldap_param.field_map)
 	{
-	  int d;
-	  mutil_parse_field_map (default_field_map, &ldap_param.field_map, &d);
+	  struct mu_config_value val;
+	  val.type = MU_CFG_STRING;
+	  val.v.string = default_field_map;
+	  if (mu_cfg_field_map (&val, &ldap_param.field_map, NULL))
+	    abort ();
 	}
     }
 }
@@ -440,9 +442,9 @@ _construct_attr_array (size_t *pargc, char ***pargv)
   for (i = 0, mu_iterator_first (itr); !mu_iterator_is_done (itr);
        mu_iterator_next (itr), i++)
     {
-      char **str;
+      char *str;
       mu_iterator_current (itr, (void**) &str); 
-      if ((argv[i] = strdup (*str)) == NULL)
+      if ((argv[i] = strdup (str)) == NULL)
         {
           mu_argcv_free (i, argv);
           return ENOMEM;
@@ -536,8 +538,6 @@ _mu_entry_to_auth_data (LDAP *ld, LDAPMessage *msg,
   
   rc = ldap_get_dn_ber (ld, msg, &ber, &bv);
   ufn = ldap_dn2ufn (bv.bv_val);
-  /* FIXME: Use debug or diag functions */
-  mu_error ("INFO: %s", ufn);
   ldap_memfree (ufn);
   
   mu_assoc_get_iterator (ldap_param.field_map, &itr);
@@ -545,17 +545,15 @@ _mu_entry_to_auth_data (LDAP *ld, LDAPMessage *msg,
        mu_iterator_next (itr))
     {
       char *key;
-      char **pattr;
       char *attr;
       struct berval **values;
       
-      mu_iterator_current_kv (itr, (const void **)&key, (void**) &pattr);
-      attr = *pattr;
+      mu_iterator_current_kv (itr, (const void **)&key, (void**) &attr);
       values = ldap_get_values_len (ld, msg, attr);
       if (!values || !values[0])
 	{
 	  mu_error ("LDAP field `%s' (`%s') has NULL value",
-		    key, *pattr);
+		    key, attr);
 	  _free_partial_auth_data (&d);
 	  return MU_ERR_READ;
 	}
@@ -598,38 +596,34 @@ _mu_ldap_search (LDAP *ld, const char *filter_pat, const char *key,
   size_t nattrs;
   LDAPMessage *res, *msg;
   ber_int_t msgid;
-  const char *env[3];
-  struct mu_wordsplit ws;
-
+  char *filter_str;
+  
   rc = _construct_attr_array (&nattrs, &attrs);
   if (rc)
     return rc;
 
-  env[0] = "user";
-  env[1] = key;
-  env[2] = NULL;
-
-  ws.ws_env = env;
-  if (mu_wordsplit (filter_pat, &ws,
-		    MU_WRDSF_NOSPLIT | MU_WRDSF_NOCMD |
-		    MU_WRDSF_ENV | MU_WRDSF_ENV_KV))
+  rc = mu_str_vexpand (&filter_str, filter_pat, "user", key, NULL);
+  if (rc)
     {
-      mu_error (_("cannot expand line `%s': %s"), filter_pat,
-		mu_wordsplit_strerror (&ws));
-      return MU_ERR_FAILURE;
-    }
-  else if (ws.ws_wordc == 0)
-    {
-      mu_error (_("expanding %s yields empty string"), filter_pat);
-      mu_wordsplit_free (&ws);
       mu_argcv_free (nattrs, attrs);
-      return MU_ERR_FAILURE;
+      if (rc == MU_ERR_FAILURE)
+	{
+	  mu_error (_("cannot expand line `%s': %s"), filter_pat,
+		    filter_str);
+	  free (filter_str);
+	}
+      else
+	{
+	  mu_error (_("cannot expand line `%s': %s"), filter_pat,
+		    mu_strerror (rc));
+	}
+      return rc;
     }
   
   rc = ldap_search_ext (ld, ldap_param.base, LDAP_SCOPE_SUBTREE,
-			ws.ws_wordv[0], attrs, 0,
+			filter_str, attrs, 0,
 			NULL, NULL, NULL, -1, &msgid);
-  mu_wordsplit_free (&ws);
+  free (filter_str);
   mu_argcv_free (nattrs, attrs);
 
   if (rc != LDAP_SUCCESS)
@@ -641,7 +635,7 @@ _mu_ldap_search (LDAP *ld, const char *filter_pat, const char *key,
 	return MU_ERR_FAILURE;
     }
 
-  rc = ldap_result (ld, msgid, LDAP_MSG_ALL, NULL, &res );
+  rc = ldap_result (ld, msgid, LDAP_MSG_ALL, NULL, &res);
   if (rc < 0)
     {
       mu_error ("ldap_result failed");
