@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2001, 2005-2007, 2010-2012, 2014-2015 Free
+   Copyright (C) 1999-2001, 2005-2007, 2010-2012, 2014-2017 Free
    Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
@@ -56,7 +56,7 @@ _get_address_part (const char *func_name, address_get_fp fun,
   mu_address_destroy (&addr);
 
   if (status == 0)
-    ret = scm_from_locale_string (str);
+    ret = scm_from_locale_string (mu_prstr (str));
   else
     {
       free (str);
