@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2012, 2014-2015 Free Software Foundation, Inc.
+   Copyright (C) 2012, 2014-2017 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
@@ -31,34 +31,16 @@
 /* Syntax: addheader [:last] <field-name: string> <value: string>
  */
 int
-sieve_addheader (mu_sieve_machine_t mach, mu_list_t args, mu_list_t tags)
+sieve_addheader (mu_sieve_machine_t mach)
 {
-  mu_sieve_value_t *val;
   const char *field_name;
   const char *field_value;
   mu_message_t msg;
   mu_header_t hdr;
   int rc;
   
-  val = mu_sieve_value_get (args, 0);
-  if (!val)
-    {
-      mu_sieve_error (mach, "%lu: %s",
-		      (unsigned long) mu_sieve_get_message_num (mach),
-		      _("cannot get field name"));
-      mu_sieve_abort (mach);
-    }
-  field_name = val->v.string;
-
-  val = mu_sieve_value_get (args, 1);
-  if (!val)
-    {
-      mu_sieve_error (mach, "%lu: %s",
-		      (unsigned long) mu_sieve_get_message_num (mach),
-		      _("cannot get field value"));
-      mu_sieve_abort (mach);
-    }
-  field_value = val->v.string;
+  mu_sieve_get_arg (mach, 0, SVT_STRING, &field_name);
+  mu_sieve_get_arg (mach, 1, SVT_STRING, &field_value);
 
   mu_sieve_log_action (mach, "ADDHEADER", "%s: %s", field_name, field_value);
 
@@ -76,7 +58,7 @@ sieve_addheader (mu_sieve_machine_t mach, mu_list_t args, mu_list_t tags)
       mu_sieve_abort (mach);
     }
 
-  rc = (mu_sieve_tag_lookup (tags, "last", NULL) ?
+  rc = (mu_sieve_get_tag (mach, "last", SVT_VOID, NULL) ?
 	mu_header_append : mu_header_prepend) (hdr, field_name, field_value);
   if (rc)
     {
@@ -95,64 +77,22 @@ sieve_addheader (mu_sieve_machine_t mach, mu_list_t args, mu_list_t tags)
                         [<value-patterns: string-list>]
  */
 int
-sieve_deleteheader (mu_sieve_machine_t mach, mu_list_t args, mu_list_t tags)
+sieve_deleteheader (mu_sieve_machine_t mach)
 {
   mu_sieve_value_t *val;
   const char *field_name;
-  const char *field_pattern;
   mu_message_t msg;
   mu_header_t hdr;
   int rc;
   mu_sieve_comparator_t comp;
   mu_iterator_t itr;
-  unsigned long i, idx = 0;
+  size_t i, idx = 0;
   
-  val = mu_sieve_value_get (args, 0);
-  if (!val)
-    {
-      mu_sieve_error (mach, "%lu: %s",
-		      (unsigned long) mu_sieve_get_message_num (mach),
-		      _("cannot get field name"));
-      mu_sieve_abort (mach);
-    }
-  field_name = val->v.string;
+  mu_sieve_get_arg (mach, 0, SVT_STRING, &field_name);
+  val = mu_sieve_get_arg_optional (mach, 1);
 
-  val = mu_sieve_value_get (args, 1);
-  if (!val)
-    {
-      field_pattern = NULL;
-      mu_sieve_log_action (mach, "DELETEHEADER", "%s", field_name);
-    }
-  else
-    {
-      switch (val->type)
-	{
-	case SVT_STRING_LIST:
-	  if (mu_list_get (val->v.list, 0, (void**)&field_pattern))
-	    {
-	      mu_sieve_error (mach, "%lu: %s",
-			      (unsigned long) mu_sieve_get_message_num (mach),
-			      _("cannot get list item"));
-	      mu_sieve_abort (mach);
-	    }
-	  mu_sieve_log_action (mach, "DELETEHEADER", "%s: (regexp)",
-			       field_name);
-	  break;
-	  
-	case SVT_STRING:
-	  field_pattern = val->v.string;
-	  mu_sieve_log_action (mach, "DELETEHEADER", "%s: %s", field_name,
-			       field_pattern);
-	  break;
-
-	default:
-	  mu_sieve_error (mach, "%lu: %s: %d",
-			  (unsigned long) mu_sieve_get_message_num (mach),
-			  _("unexpected value type"), val->type);
-	  mu_sieve_abort (mach);
-	  
-	}
-    }
+  mu_sieve_log_action (mach, "DELETEHEADER", "%s%s", field_name,
+		       val ? " (values)" : "" );
   
   if (mu_sieve_is_dry_run (mach))
     return 0;
@@ -168,16 +108,22 @@ sieve_deleteheader (mu_sieve_machine_t mach, mu_list_t args, mu_list_t tags)
       mu_sieve_abort (mach);
     }
 
-  mu_header_get_iterator (hdr, &itr);
-  if (mu_sieve_tag_lookup (tags, "last", NULL))
+  rc = mu_header_get_iterator (hdr, &itr);
+  if (rc)
+    {
+      mu_sieve_error (mach, "mu_header_get_iterator: %s",
+		      mu_strerror (rc));
+      mu_sieve_abort (mach);
+    }
+      
+  if (mu_sieve_get_tag (mach, "last", SVT_VOID, NULL))
     {
       int backwards = 1;
       mu_iterator_ctl (itr, mu_itrctl_set_direction, &backwards);
     }
-  comp = mu_sieve_get_comparator (mach, tags);
+  comp = mu_sieve_get_comparator (mach);
 
-  if (mu_sieve_tag_lookup (tags, "index", &val))
-    idx = val->v.number;
+  mu_sieve_get_tag (mach, "index", SVT_NUMBER, &idx);
   
   for (i = 0, mu_iterator_first (itr); !mu_iterator_is_done (itr);
        mu_iterator_next (itr))
@@ -190,10 +136,18 @@ sieve_deleteheader (mu_sieve_machine_t mach, mu_list_t args, mu_list_t tags)
       if (idx && ++i < idx)
 	continue;
 	  
-      if (field_pattern)
+      if (val)
 	{
-	  if (comp (field_pattern, fv))
-	    mu_iterator_ctl (itr, mu_itrctl_delete, NULL);
+	  for (i = 0; i < val->v.list.count; i++)
+	    {
+	      mu_sieve_string_t *s = mu_sieve_string_raw (mach,
+							  &val->v.list, i);
+	      if (comp (mach, s, fv))
+		{
+		  mu_iterator_ctl (itr, mu_itrctl_delete, NULL);
+		  break;
+		}
+	    }
 	}
       else
 	mu_iterator_ctl (itr, mu_itrctl_delete, NULL);
@@ -318,22 +272,13 @@ static mu_sieve_data_type deleteheader_args[] = {
 int
 SIEVE_EXPORT (editheader, init) (mu_sieve_machine_t mach)
 {
-  int rc;
-
   /* This dummy record is required by libmu_sieve  */
-  rc = mu_sieve_register_action (mach, "editheader", NULL, NULL, NULL, 1);
-  if (rc)
-    return rc;
-  rc = mu_sieve_register_action (mach, "addheader", sieve_addheader,
-				 addheader_args, addheader_tag_groups, 1);
-  if (rc)
-    return rc;
-  rc = mu_sieve_register_action_ext (mach, "deleteheader", sieve_deleteheader,
-				     deleteheader_args, deleteheader_args,
-				     deleteheader_tag_groups,
-				     1);
-  if (rc)
-    return rc;
-
-  return rc;
+  mu_sieve_register_action (mach, "editheader", NULL, NULL, NULL, 1);
+  mu_sieve_register_action (mach, "addheader", sieve_addheader,
+			    addheader_args, addheader_tag_groups, 1);
+  mu_sieve_register_action_ext (mach, "deleteheader", sieve_deleteheader,
+				deleteheader_args, deleteheader_args,
+				deleteheader_tag_groups,
+				1);
+  return 0;
 }
