@@ -1,4 +1,4 @@
-/* A Bison parser, made by GNU Bison 2.7.  */
+/* A Bison parser, made by GNU Bison 2.5.1.  */
 
 /* Bison interface for Yacc-like parsers in C
    
@@ -30,22 +30,13 @@
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
-#ifndef YY_YY_Y_TAB_H_INCLUDED
-# define YY_YY_Y_TAB_H_INCLUDED
-/* Enabling traces.  */
-#ifndef YYDEBUG
-# define YYDEBUG 1
-#endif
-#if YYDEBUG
-extern int mu_sieve_yydebug;
-#endif
 
 /* Tokens.  */
 #ifndef YYTOKENTYPE
 # define YYTOKENTYPE
    /* Put the tokens into the symbol table, so that GDB and other debuggers
       know about them.  */
-   enum mu_sieve_yytokentype {
+   enum yytokentype {
      IDENT = 258,
      TAG = 259,
      NUMBER = 260,
@@ -80,11 +71,13 @@ extern int mu_sieve_yydebug;
 
 
 
+
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
 {
-/* Line 2058 of yacc.c  */
-#line 78 "sieve.y"
+
+/* Line 2072 of yacc.c  */
+#line 44 "sieve-gram.y"
 
   char *string;
   size_t number;
@@ -93,24 +86,25 @@ typedef union YYSTYPE
   struct
   {
     char *ident;
+    struct mu_locus_range idloc;
     size_t first;
     size_t count;
   } command;
-  struct node_list
-  {
-    struct mu_sieve_node *head, *tail;
-  } node_list;
+  struct mu_sieve_node_list node_list;
   struct mu_sieve_node *node;
 
 
-/* Line 2058 of yacc.c  */
-#line 108 "sieve-gram.h"
+
+/* Line 2072 of yacc.c  */
+#line 100 "sieve-gram.h"
 } YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
-# define mu_sieve_yystype YYSTYPE /* obsolescent; will be withdrawn */
+# define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
 #endif
 
+extern YYSTYPE mu_sieve_yylval;
+
 #if ! defined YYLTYPE && ! defined YYLTYPE_IS_DECLARED
 typedef struct YYLTYPE
 {
@@ -119,25 +113,10 @@ typedef struct YYLTYPE
   int last_line;
   int last_column;
 } YYLTYPE;
-# define mu_sieve_yyltype YYLTYPE /* obsolescent; will be withdrawn */
+# define yyltype YYLTYPE /* obsolescent; will be withdrawn */
 # define YYLTYPE_IS_DECLARED 1
 # define YYLTYPE_IS_TRIVIAL 1
 #endif
 
-extern YYSTYPE mu_sieve_yylval;
 extern YYLTYPE mu_sieve_yylloc;
-#ifdef YYPARSE_PARAM
-#if defined __STDC__ || defined __cplusplus
-int mu_sieve_yyparse (void *YYPARSE_PARAM);
-#else
-int mu_sieve_yyparse ();
-#endif
-#else /* ! YYPARSE_PARAM */
-#if defined __STDC__ || defined __cplusplus
-int mu_sieve_yyparse (void);
-#else
-int mu_sieve_yyparse ();
-#endif
-#endif /* ! YYPARSE_PARAM */
 
-#endif /* !YY_YY_Y_TAB_H_INCLUDED  */
