@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001-2002, 2005-2007, 2009-2012, 2014-2016 Free
+   Copyright (C) 1999, 2001-2002, 2005-2007, 2009-2012, 2014-2017 Free
    Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
@@ -273,7 +273,7 @@ escape_run_editor (char *ed, int argc, char **argv, compose_env_t *env)
     }
 
   mu_stream_seek (env->compstr, 0, MU_SEEK_SET, NULL);
-  if (!mailvar_get (NULL, "editheaders", mailvar_type_boolean, 0))
+  if (mailvar_is_true ("editheaders"))
     {
       dump_headers (tempstream, env);
 
