@@ -1,4 +1,4 @@
-/* A Bison parser, made by GNU Bison 2.7.  */
+/* A Bison parser, made by GNU Bison 2.5.1.  */
 
 /* Bison implementation for Yacc-like parsers in C
    
@@ -44,7 +44,7 @@
 #define YYBISON 1
 
 /* Bison version.  */
-#define YYBISON_VERSION "2.7"
+#define YYBISON_VERSION "2.5.1"
 
 /* Skeleton name.  */
 #define YYSKELETON_NAME "yacc.c"
@@ -58,11 +58,14 @@
 /* Pull parsers.  */
 #define YYPULL 1
 
+/* Using locations.  */
+#define YYLSP_NEEDED 0
 
 
 
 /* Copy the first part of user declarations.  */
-/* Line 371 of yacc.c  */
+
+/* Line 268 of yacc.c  */
 #line 18 "msgset.y"
 
 #include "mail.h"
@@ -101,8 +104,9 @@ int yylex  (void);
 static int msgset_flags = MSG_NODELETED;
 static msgset_t *result;
 
-/* Line 371 of yacc.c  */
-#line 106 "msgset.c"
+
+/* Line 268 of yacc.c  */
+#line 110 "msgset.c"
 
 # ifndef YY_NULL
 #  if defined __cplusplus && 201103L <= __cplusplus
@@ -112,6 +116,11 @@ static msgset_t *result;
 #  endif
 # endif
 
+/* Enabling traces.  */
+#ifndef YYDEBUG
+# define YYDEBUG 1
+#endif
+
 /* Enabling verbose error messages.  */
 #ifdef YYERROR_VERBOSE
 # undef YYERROR_VERBOSE
@@ -120,14 +129,11 @@ static msgset_t *result;
 # define YYERROR_VERBOSE 0
 #endif
 
+/* Enabling the token table.  */
+#ifndef YYTOKEN_TABLE
+# define YYTOKEN_TABLE 0
+#endif
 
-/* Enabling traces.  */
-#ifndef YYDEBUG
-# define YYDEBUG 1
-#endif
-#if YYDEBUG
-extern int yydebug;
-#endif
 
 /* Tokens.  */
 #ifndef YYTOKENTYPE
@@ -153,10 +159,12 @@ extern int yydebug;
 
 
 
+
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
 {
-/* Line 387 of yacc.c  */
+
+/* Line 295 of yacc.c  */
 #line 56 "msgset.y"
 
   char *string;
@@ -165,36 +173,21 @@ typedef union YYSTYPE
   msgset_t *mset;
 
 
-/* Line 387 of yacc.c  */
-#line 170 "msgset.c"
+
+/* Line 295 of yacc.c  */
+#line 179 "msgset.c"
 } YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
 #endif
 
-extern YYSTYPE yylval;
-
-#ifdef YYPARSE_PARAM
-#if defined __STDC__ || defined __cplusplus
-int yyparse (void *YYPARSE_PARAM);
-#else
-int yyparse ();
-#endif
-#else /* ! YYPARSE_PARAM */
-#if defined __STDC__ || defined __cplusplus
-int yyparse (void);
-#else
-int yyparse ();
-#endif
-#endif /* ! YYPARSE_PARAM */
-
-
 
 /* Copy the second part of user declarations.  */
 
-/* Line 390 of yacc.c  */
-#line 198 "msgset.c"
+
+/* Line 345 of yacc.c  */
+#line 191 "msgset.c"
 
 #ifdef short
 # undef short
@@ -247,24 +240,24 @@ typedef short int yytype_int16;
 # if defined YYENABLE_NLS && YYENABLE_NLS
 #  if ENABLE_NLS
 #   include <libintl.h> /* INFRINGES ON USER NAME SPACE */
-#   define YY_(Msgid) dgettext ("bison-runtime", Msgid)
+#   define YY_(msgid) dgettext ("bison-runtime", msgid)
 #  endif
 # endif
 # ifndef YY_
-#  define YY_(Msgid) Msgid
+#  define YY_(msgid) msgid
 # endif
 #endif
 
 /* Suppress unused-variable warnings by "using" E.  */
 #if ! defined lint || defined __GNUC__
-# define YYUSE(E) ((void) (E))
+# define YYUSE(e) ((void) (e))
 #else
-# define YYUSE(E) /* empty */
+# define YYUSE(e) /* empty */
 #endif
 
 /* Identity function, used to suppress warnings about constant conditions.  */
 #ifndef lint
-# define YYID(N) (N)
+# define YYID(n) (n)
 #else
 #if (defined __STDC__ || defined __C99__FUNC__ \
      || defined __cplusplus || defined _MSC_VER)
@@ -499,7 +492,7 @@ static const yytype_uint8 yyrline[] =
 };
 #endif
 
-#if YYDEBUG || YYERROR_VERBOSE || 0
+#if YYDEBUG || YYERROR_VERBOSE || YYTOKEN_TABLE
 /* YYTNAME[SYMBOL-NUM] -- String name of the symbol SYMBOL-NUM.
    First, the terminals, then, starting at YYNTOKENS, nonterminals.  */
 static const char *const yytname[] =
@@ -596,10 +589,10 @@ static const yytype_int8 yytable[] =
       39,    42,     0,    14,    40,    47,    14,    14
 };
 
-#define yypact_value_is_default(Yystate) \
-  (!!((Yystate) == (-15)))
+#define yypact_value_is_default(yystate) \
+  ((yystate) == (-15))
 
-#define yytable_value_is_error(Yytable_value) \
+#define yytable_value_is_error(yytable_value) \
   YYID (0)
 
 static const yytype_int8 yycheck[] =
@@ -670,18 +663,46 @@ do                                                              \
     }								\
 while (YYID (0))
 
-/* Error token number */
+
 #define YYTERROR	1
 #define YYERRCODE	256
 
 
+/* YYLLOC_DEFAULT -- Set CURRENT to span from RHS[1] to RHS[N].
+   If N is 0, then set CURRENT to the empty location which ends
+   the previous symbol: RHS[0] (always defined).  */
+
+#define YYRHSLOC(Rhs, K) ((Rhs)[K])
+#ifndef YYLLOC_DEFAULT
+# define YYLLOC_DEFAULT(Current, Rhs, N)				\
+    do									\
+      if (YYID (N))                                                    \
+	{								\
+	  (Current).first_line   = YYRHSLOC (Rhs, 1).first_line;	\
+	  (Current).first_column = YYRHSLOC (Rhs, 1).first_column;	\
+	  (Current).last_line    = YYRHSLOC (Rhs, N).last_line;		\
+	  (Current).last_column  = YYRHSLOC (Rhs, N).last_column;	\
+	}								\
+      else								\
+	{								\
+	  (Current).first_line   = (Current).last_line   =		\
+	    YYRHSLOC (Rhs, 0).last_line;				\
+	  (Current).first_column = (Current).last_column =		\
+	    YYRHSLOC (Rhs, 0).last_column;				\
+	}								\
+    while (YYID (0))
+#endif
+
+
 /* This macro is provided for backward compatibility. */
+
 #ifndef YY_LOCATION_PRINT
 # define YY_LOCATION_PRINT(File, Loc) ((void) 0)
 #endif
 
 
 /* YYLEX -- calling `yylex' with the right arguments.  */
+
 #ifdef YYLEX_PARAM
 # define YYLEX yylex (YYLEX_PARAM)
 #else
@@ -744,7 +765,7 @@ yy_symbol_value_print (yyoutput, yytype, yyvaluep)
   switch (yytype)
     {
       default:
-        break;
+	break;
     }
 }
 
@@ -986,6 +1007,7 @@ yysyntax_error (YYSIZE_T *yymsg_alloc, char **yymsg,
 {
   YYSIZE_T yysize0 = yytnamerr (YY_NULL, yytname[yytoken]);
   YYSIZE_T yysize = yysize0;
+  YYSIZE_T yysize1;
   enum { YYERROR_VERBOSE_ARGS_MAXIMUM = 5 };
   /* Internationalized format string. */
   const char *yyformat = YY_NULL;
@@ -1048,13 +1070,11 @@ yysyntax_error (YYSIZE_T *yymsg_alloc, char **yymsg,
                     break;
                   }
                 yyarg[yycount++] = yytname[yyx];
-                {
-                  YYSIZE_T yysize1 = yysize + yytnamerr (YY_NULL, yytname[yyx]);
-                  if (! (yysize <= yysize1
-                         && yysize1 <= YYSTACK_ALLOC_MAXIMUM))
-                    return 2;
-                  yysize = yysize1;
-                }
+                yysize1 = yysize + yytnamerr (YY_NULL, yytname[yyx]);
+                if (! (yysize <= yysize1
+                       && yysize1 <= YYSTACK_ALLOC_MAXIMUM))
+                  return 2;
+                yysize = yysize1;
               }
         }
     }
@@ -1074,12 +1094,10 @@ yysyntax_error (YYSIZE_T *yymsg_alloc, char **yymsg,
 # undef YYCASE_
     }
 
-  {
-    YYSIZE_T yysize1 = yysize + yystrlen (yyformat);
-    if (! (yysize <= yysize1 && yysize1 <= YYSTACK_ALLOC_MAXIMUM))
-      return 2;
-    yysize = yysize1;
-  }
+  yysize1 = yysize + yystrlen (yyformat);
+  if (! (yysize <= yysize1 && yysize1 <= YYSTACK_ALLOC_MAXIMUM))
+    return 2;
+  yysize = yysize1;
 
   if (*yymsg_alloc < yysize)
     {
@@ -1139,27 +1157,32 @@ yydestruct (yymsg, yytype, yyvaluep)
     {
 
       default:
-        break;
+	break;
     }
 }
 
 
+/* Prevent warnings from -Wmissing-prototypes.  */
+#ifdef YYPARSE_PARAM
+#if defined __STDC__ || defined __cplusplus
+int yyparse (void *YYPARSE_PARAM);
+#else
+int yyparse ();
+#endif
+#else /* ! YYPARSE_PARAM */
+#if defined __STDC__ || defined __cplusplus
+int yyparse (void);
+#else
+int yyparse ();
+#endif
+#endif /* ! YYPARSE_PARAM */
 
 
 /* The lookahead symbol.  */
 int yychar;
 
-
-#ifndef YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
-# define YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
-# define YY_IGNORE_MAYBE_UNINITIALIZED_END
-#endif
-#ifndef YY_INITIAL_VALUE
-# define YY_INITIAL_VALUE(Value) /* Nothing. */
-#endif
-
 /* The semantic value of the lookahead symbol.  */
-YYSTYPE yylval YY_INITIAL_VALUE(yyval_default);
+YYSTYPE yylval;
 
 /* Number of syntax errors so far.  */
 int yynerrs;
@@ -1217,7 +1240,7 @@ yyparse ()
   int yyn;
   int yyresult;
   /* Lookahead token as an internal (translated) token number.  */
-  int yytoken = 0;
+  int yytoken;
   /* The variables used to return semantic value and location from the
      action routines.  */
   YYSTYPE yyval;
@@ -1235,8 +1258,9 @@ yyparse ()
      Keep to zero when no symbol should be popped.  */
   int yylen = 0;
 
-  yyssp = yyss = yyssa;
-  yyvsp = yyvs = yyvsa;
+  yytoken = 0;
+  yyss = yyssa;
+  yyvs = yyvsa;
   yystacksize = YYINITDEPTH;
 
   YYDPRINTF ((stderr, "Starting parse\n"));
@@ -1245,6 +1269,14 @@ yyparse ()
   yyerrstatus = 0;
   yynerrs = 0;
   yychar = YYEMPTY; /* Cause a token to be read.  */
+
+  /* Initialize stack pointers.
+     Waste one element of value and location stack
+     so that they stay on the same level as the state stack.
+     The wasted elements are never initialized.  */
+  yyssp = yyss;
+  yyvsp = yyvs;
+
   goto yysetstate;
 
 /*------------------------------------------------------------.
@@ -1385,9 +1417,7 @@ yybackup:
   yychar = YYEMPTY;
 
   yystate = yyn;
-  YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
   *++yyvsp = yylval;
-  YY_IGNORE_MAYBE_UNINITIALIZED_END
 
   goto yynewstate;
 
@@ -1424,7 +1454,8 @@ yyreduce:
   switch (yyn)
     {
         case 2:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 72 "msgset.y"
     {
 	     result = msgset_make_1 (get_cursor ());
@@ -1432,7 +1463,8 @@ yyreduce:
     break;
 
   case 3:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 76 "msgset.y"
     {
 	     result = msgset_make_1 (get_cursor ());
@@ -1440,7 +1472,8 @@ yyreduce:
     break;
 
   case 4:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 80 "msgset.y"
     {
 	     result = (yyvsp[(1) - (1)].mset);
@@ -1448,7 +1481,8 @@ yyreduce:
     break;
 
   case 5:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 84 "msgset.y"
     {
 	     result = msgset_select (select_deleted, NULL, 0, 1);
@@ -1456,7 +1490,8 @@ yyreduce:
     break;
 
   case 6:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 88 "msgset.y"
     {
 	     result = msgset_select (select_deleted, NULL, 1, 1);
@@ -1464,7 +1499,8 @@ yyreduce:
     break;
 
   case 7:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 92 "msgset.y"
     {
 	     result = msgset_select (select_deleted, NULL, 0, total);
@@ -1472,7 +1508,8 @@ yyreduce:
     break;
 
   case 8:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 96 "msgset.y"
     {
 	     result = msgset_select (select_deleted, NULL, 1, 1);
@@ -1480,7 +1517,8 @@ yyreduce:
     break;
 
   case 9:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 100 "msgset.y"
     {
 	     result = msgset_select (select_deleted, NULL, 0, 1);
@@ -1488,7 +1526,8 @@ yyreduce:
     break;
 
   case 11:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 107 "msgset.y"
     {
 	     (yyval.mset) = msgset_append ((yyvsp[(1) - (3)].mset), (yyvsp[(3) - (3)].mset));
@@ -1496,7 +1535,8 @@ yyreduce:
     break;
 
   case 12:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 111 "msgset.y"
     {
 	     (yyval.mset) = msgset_append ((yyvsp[(1) - (2)].mset), (yyvsp[(2) - (2)].mset));
@@ -1504,7 +1544,8 @@ yyreduce:
     break;
 
   case 13:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 117 "msgset.y"
     {
 	     (yyval.mset) = (yyvsp[(1) - (1)].mset);
@@ -1514,7 +1555,8 @@ yyreduce:
     break;
 
   case 14:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 123 "msgset.y"
     {
 	     (yyval.mset) = (yyvsp[(2) - (3)].mset);
@@ -1522,7 +1564,8 @@ yyreduce:
     break;
 
   case 15:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 127 "msgset.y"
     {
 	     (yyval.mset) = msgset_negate ((yyvsp[(2) - (2)].mset));
@@ -1530,7 +1573,8 @@ yyreduce:
     break;
 
   case 17:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 134 "msgset.y"
     {
 	     (yyval.mset) = msgset_expand ((yyvsp[(1) - (4)].mset), (yyvsp[(3) - (4)].mset));
@@ -1540,7 +1584,8 @@ yyreduce:
     break;
 
   case 19:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 143 "msgset.y"
     {
 	     struct header_data hd;
@@ -1554,7 +1599,8 @@ yyreduce:
     break;
 
   case 20:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 153 "msgset.y"
     {
 	     (yyval.mset) = msgset_select (select_body, (yyvsp[(1) - (1)].string), 0, 0);
@@ -1562,7 +1608,8 @@ yyreduce:
     break;
 
   case 21:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 157 "msgset.y"
     {
 	     if (strchr ("dnorTtu", (yyvsp[(1) - (1)].type)) == NULL)
@@ -1575,7 +1622,8 @@ yyreduce:
     break;
 
   case 22:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 166 "msgset.y"
     {
 	     (yyval.mset) = msgset_select (select_sender, (void *)(yyvsp[(1) - (1)].string), 0, 0);
@@ -1584,7 +1632,8 @@ yyreduce:
     break;
 
   case 23:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 173 "msgset.y"
     {
 	     (yyval.string) = NULL;
@@ -1592,7 +1641,8 @@ yyreduce:
     break;
 
   case 24:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 177 "msgset.y"
     {
 	     (yyval.string) = (yyvsp[(1) - (1)].string);
@@ -1600,7 +1650,8 @@ yyreduce:
     break;
 
   case 26:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 184 "msgset.y"
     {
 	     (yyval.mset) = msgset_append ((yyvsp[(1) - (3)].mset), (yyvsp[(3) - (3)].mset));
@@ -1608,7 +1659,8 @@ yyreduce:
     break;
 
   case 27:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 188 "msgset.y"
     {
 	     (yyval.mset) = msgset_append ((yyvsp[(1) - (2)].mset), (yyvsp[(2) - (2)].mset));
@@ -1616,7 +1668,8 @@ yyreduce:
     break;
 
   case 29:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 195 "msgset.y"
     {
 	     if ((yyvsp[(3) - (3)].mset)->npart == 1)
@@ -1634,7 +1687,8 @@ yyreduce:
     break;
 
   case 30:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 209 "msgset.y"
     {
 	     (yyval.mset) = msgset_range ((yyvsp[(1) - (3)].number), total);
@@ -1642,7 +1696,8 @@ yyreduce:
     break;
 
   case 32:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 216 "msgset.y"
     {
 	     (yyval.mset) = msgset_expand ((yyvsp[(1) - (4)].mset), (yyvsp[(3) - (4)].mset));
@@ -1652,7 +1707,8 @@ yyreduce:
     break;
 
   case 33:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 224 "msgset.y"
     {
 	     (yyval.mset) = msgset_make_1 ((yyvsp[(1) - (1)].number));
@@ -1660,7 +1716,8 @@ yyreduce:
     break;
 
   case 34:
-/* Line 1792 of yacc.c  */
+
+/* Line 1810 of yacc.c  */
 #line 228 "msgset.y"
     {
 	     (yyval.mset) = (yyvsp[(2) - (3)].mset);
@@ -1668,8 +1725,9 @@ yyreduce:
     break;
 
 
-/* Line 1792 of yacc.c  */
-#line 1673 "msgset.c"
+
+/* Line 1810 of yacc.c  */
+#line 1731 "msgset.c"
       default: break;
     }
   /* User semantic actions sometimes alter yychar, and that requires
@@ -1832,9 +1890,7 @@ yyerrlab1:
       YY_STACK_PRINT (yyss, yyssp);
     }
 
-  YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
   *++yyvsp = yylval;
-  YY_IGNORE_MAYBE_UNINITIALIZED_END
 
 
   /* Shift the error token.  */
@@ -1900,7 +1956,8 @@ yyreturn:
 }
 
 
-/* Line 2055 of yacc.c  */
+
+/* Line 2071 of yacc.c  */
 #line 232 "msgset.y"
 
 
@@ -2447,3 +2504,4 @@ main(int argc, char **argv)
   return 0;
 }
 #endif
+
