@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001-2012, 2014-2016 Free Software Foundation,
+   Copyright (C) 1999, 2001-2012, 2014-2017 Free Software Foundation,
    Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
@@ -24,6 +24,8 @@
 static int isfilename (const char *);
 static int msg_to_pipe (const char *cmd, mu_message_t msg);
 
+int multipart_alternative;
+
 
 /* Additional message headers */
 struct add_header
@@ -134,55 +136,160 @@ mail_sendheader (int argc, char **argv)
 /* Attachments */
 struct atchinfo
 {
+  char *id;
   char *encoding;
   char *content_type;
+  char *name;
   char *filename;
+  mu_stream_t source;
+  int skip_empty;
 };
 
-static mu_list_t attlist;
-
-char *default_encoding;
-char *default_content_type;
-
 static void
 atchinfo_free (void *p)
 {
   struct atchinfo *ap = p;
+  free (ap->id);
   free (ap->encoding);
   free (ap->content_type);
+  free (ap->name);
   free (ap->filename);
+  mu_stream_destroy (&ap->source);
   free (ap);
 }
 
+static mu_list_t
+attlist_new (void)
+{
+  mu_list_t lst;
+  int rc = mu_list_create (&lst);
+  if (rc)
+    {
+      mu_diag_funcall (MU_DIAG_ERROR, "mu_list_create", NULL, rc);
+      exit (1);
+    }
+  mu_list_set_destroy_item (lst, atchinfo_free);
+  return lst;
+}
+
+static void
+attach_set_content_type (struct atchinfo *aptr, char const *content_type)
+{
+  char *charset;
+
+  if (!content_type)
+    content_type = "text/plain";
+  if (strncmp (content_type, "text/", 5) == 0
+      && !strstr (content_type, "charset=")
+      && (charset = util_get_charset ()))
+    {
+      mu_asprintf (&aptr->content_type, "%s; charset=%s",
+		   content_type, charset);
+      free (charset);
+    }
+  else
+    aptr->content_type = mu_strdup (content_type);
+}
+
+static void
+attlist_add (mu_list_t attlist, char *id, char const *encoding,
+	     char const *content_type, char const *content_name,
+	     char const *content_filename,
+	     mu_stream_t stream, int skip_empty)
+{
+  struct atchinfo *aptr;
+  int rc;
+  
+  aptr = mu_alloc (sizeof (*aptr));
+
+  aptr->id = id ? mu_strdup (id) : id;
+  aptr->encoding = mu_strdup (encoding);
+  attach_set_content_type (aptr,
+			   content_type
+			     ? content_type : "application/octet-stream");
+  aptr->name = content_name ? mu_strdup (content_name) : NULL;
+  aptr->filename = content_filename ? mu_strdup (content_filename) : NULL;
+  aptr->source = stream;
+  if (stream)
+    mu_stream_ref (stream);
+  aptr->skip_empty = skip_empty;
+  rc = mu_list_append (attlist, aptr);
+  if (rc)
+    {
+      mu_diag_funcall (MU_DIAG_ERROR, "mu_list_append", NULL, rc);
+      exit (1);
+    }
+}
+
 int
-send_attach_file (const char *name,
-		  const char *content_type, const char *encoding)
+attlist_attach_file (mu_list_t *attlist_ptr,
+		     int fd,
+		     const char *realname,
+		     const char *content_filename, const char *content_name,
+		     const char *content_type, const char *encoding)
 {
   int rc;
   struct stat st;
-  struct atchinfo *aptr;
   mu_list_t list;
+  mu_stream_t stream = NULL;
+  char *id = NULL;
+  mu_list_t attlist;
   
-  if (stat (name, &st))
+  if (fd >= 0)
     {
-      if (errno == ENOENT)
+      rc = mu_fd_stream_create (&stream, NULL, fd, MU_STREAM_READ);
+      if (rc)
 	{
-	  mu_error (_("%s: file does not exist"), name);
+	  mu_error (_("can't open descriptor %d: %s"), fd, mu_strerror (rc));
 	  return 1;
 	}
-      else
+      mu_asprintf (&id, "fd %d", fd);
+      if (fd == 0)
 	{
-	  mu_error (_("%s: cannot stat: %s"), name, mu_strerror (errno));
-	  return 1;
+	  mu_stream_destroy (&mu_strin);
+	  mu_nullstream_create (&mu_strin, MU_STREAM_READ);
+	  mu_stream_ioctl (mu_strin, MU_IOCTL_NULLSTREAM,
+			   MU_IOCTL_NULLSTREAM_SET_PATTERN, NULL);
+	  util_do_command ("set nullbody nullbodymsg");
 	}
     }
-
-  if (!S_ISREG (st.st_mode))
+  else if (realname)
     {
-      mu_error (_("%s: not a regular file"), name);
-      return 1;
-    }
+      if (!content_filename)
+	content_filename = realname;
+      if (stat (realname, &st))
+	{
+	  if (errno == ENOENT)
+	    {
+	      mu_error (_("%s: file does not exist"), realname);
+	      return 1;
+	    }
+	  else
+	    {
+	      mu_error (_("%s: cannot stat: %s"), realname,
+			mu_strerror (errno));
+	      return 1;
+	    }
+	}
+      
+      if (!S_ISREG (st.st_mode))
+	{
+	  mu_error (_("%s: not a regular file"), realname);
+	  return 1;
+	}
 
+      rc = mu_mapfile_stream_create (&stream, realname, MU_STREAM_READ);
+      if (rc)
+	{
+	  mu_error (_("can't open file %s: %s"),
+		    realname, mu_strerror (rc));
+	  return 1;
+	}
+      mu_asprintf (&id, "\"%s\"", realname);
+    }
+  else
+    abort ();
+  
   if (!encoding)
     encoding = "base64";
   mu_filter_get_list (&list);
@@ -190,39 +297,81 @@ send_attach_file (const char *name,
   if (rc)
     {
       mu_error (_("unsupported encoding: %s"), encoding);
+      free (id);
+      mu_stream_destroy (&stream);
       return 1;
     }
   
-  if (!attlist)
+  if (!*attlist_ptr)
     {
-      rc = mu_list_create (&attlist);
-      if (rc)
-	{
-	  mu_diag_funcall (MU_DIAG_ERROR, "mu_list_create", NULL, rc);
-	  exit (1);
-	}
-      mu_list_set_destroy_item (attlist, atchinfo_free);
+      attlist = attlist_new ();
+      *attlist_ptr = attlist;
     }
-  aptr = mu_alloc (sizeof (*aptr));
+  else
+    attlist = *attlist_ptr;
+    
+  attlist_add (attlist, id, encoding, content_type,
+	       content_name, content_filename,
+	       stream, skip_empty_attachments);
+  if (stream)
+    mu_stream_unref (stream);
+  free (id);
 
-  aptr->encoding = mu_strdup (encoding);  
-  aptr->content_type = mu_strdup (content_type ?
-				  content_type :
-				    "application/octet-stream");
-  aptr->filename = mu_strdup (name);
-  rc = mu_list_append (attlist, aptr);
-  if (rc)
-    {
-      mu_diag_funcall (MU_DIAG_ERROR, "mu_list_append", NULL, rc);
-      exit (1);
-    }
   return 0;
 }
 
-int
-send_attach_file_default (const char *name)
+static int
+attlist_helper (void *item, void *data)
 {
-  return send_attach_file (name, default_content_type, default_encoding);
+  struct atchinfo *aptr = item;
+  mu_list_t list = data;
+  attlist_add (list, aptr->id, aptr->encoding, aptr->content_type,
+	       aptr->name, aptr->filename, aptr->source, aptr->skip_empty);
+  return 0;
+}
+
+static mu_list_t
+attlist_copy (mu_list_t src)
+{
+  mu_list_t dst;
+
+  if (!src)
+    return NULL;
+  dst = attlist_new ();
+  mu_list_foreach (src, attlist_helper, dst);
+  return dst;
+}
+
+static mu_list_t attachment_list;
+
+int
+send_attach_file (int fd,
+		  const char *realname,
+		  const char *content_filename, const char *content_name,
+		  const char *content_type, const char *encoding)
+{
+  return attlist_attach_file (&attachment_list,
+			      fd,
+			      realname,
+			      content_filename,
+			      content_name,
+			      content_type,
+			      encoding);
+}
+
+static void
+report_multipart_type (compose_env_t *env)
+{
+  mu_printf ("multipart/%s\n", env->alt ? "alternative" : "mixed");
+}
+
+/* ~/ - toggle between multipart/mixed and multipart/alternative */
+int
+escape_toggle_multipart_type (int argc, char **argv, compose_env_t *env)
+{
+  env->alt = !env->alt;
+  report_multipart_type (env);
+  return 0;
 }
 
 int
@@ -231,8 +380,10 @@ escape_list_attachments (int argc, char **argv, compose_env_t *env)
   mu_iterator_t itr;
   int i;
   
-  if (mu_list_is_empty (attlist) ||
-      mu_list_get_iterator (attlist, &itr))
+  report_multipart_type (env);
+
+  if (mu_list_is_empty (env->attlist) ||
+      mu_list_get_iterator (env->attlist, &itr))
     {
       mu_printf ("%s\n", _("No attachments"));
       return 0;
@@ -246,7 +397,7 @@ escape_list_attachments (int argc, char **argv, compose_env_t *env)
 	continue;
 	  
       mu_printf ("%3d %-12s %-30s %-s\n",
-		 i, aptr->filename, aptr->content_type, aptr->encoding);
+		 i, aptr->id, aptr->content_type, aptr->encoding);
     }
   mu_iterator_destroy (&itr);
 
@@ -266,7 +417,9 @@ escape_attach (int argc, char **argv, compose_env_t *env)
     case 3:
       content_type = argv[2];
     case 2:
-      return send_attach_file (argv[1], content_type, encoding);
+      return attlist_attach_file (&env->attlist,
+				  -1, argv[1], argv[1], argv[1],
+				  content_type, encoding);
     default:
       return escape_check_args (argc, argv, 2, 4);
     }
@@ -289,43 +442,72 @@ escape_remove_attachment (int argc, char **argv, compose_env_t *env)
       return 1;
     }
   
-  mu_list_count (attlist, &count);
+  mu_list_count (env->attlist, &count);
   if (n == 0 || n > count)
     {
       mu_error (_("index out of range"));
       return 1;
     }
 
-  return mu_list_remove_nth (attlist, n - 1);
+  return mu_list_remove_nth (env->attlist, n - 1);
 }
-
+
 static int
 saveatt (void *item, void *data)
 {
   struct atchinfo *aptr = item;
-  mu_mime_t mime = data;
+  compose_env_t *env = data;
   mu_message_t part;
   mu_header_t hdr;
   int rc;
   size_t nparts;
   char *p;
-  
-  rc = mu_message_create_attachment (aptr->content_type, aptr->encoding,
-				     aptr->filename, &part);
+
+  rc = mu_attachment_create (&part, aptr->content_type, aptr->encoding,
+			     aptr->name,
+			     aptr->filename);
   if (rc)
     {
-      mu_error (_("cannot attach \"%s\": %s"), aptr->filename,
-		mu_strerror (rc));
+      mu_error (_("can't create attachment %s: %s"),
+		aptr->id, mu_strerror (rc));
       return 1;
     }
 
-  mu_mime_get_num_parts	(mime, &nparts);
+  rc = mu_attachment_copy_from_stream (part, aptr->source);
+  if (rc)
+    {
+      mu_error (_("cannot attach %s: %s"), aptr->id, mu_strerror (rc));
+      return 1;
+    }
+
+  if (aptr->skip_empty)
+    {
+      mu_body_t body;
+      size_t size;
+      
+      rc = mu_message_get_body (part, &body);
+      if (rc)
+	{
+	  mu_diag_funcall (MU_DIAG_ERROR, "mu_message_get_body", aptr->id, rc);
+	  return 1;
+	}
+      rc = mu_body_size (body, &size);
+      if (rc)
+	{
+	  mu_diag_funcall (MU_DIAG_ERROR, "mu_body_size", aptr->id, rc);
+	  return 1;
+	}
+      if (size == 0)
+	return 0;
+    }
+      
+  mu_mime_get_num_parts	(env->mime, &nparts);
   mu_message_get_header (part, &hdr);
   mu_rfc2822_msg_id (nparts, &p);
   mu_header_set_value (hdr, MU_HEADER_CONTENT_ID, p, 1);
   free (p);
 
-  rc = mu_mime_add_part (mime, part);
+  rc = mu_mime_add_part (env->mime, part);
   mu_message_unref (part);
   if (rc)
     {
@@ -337,95 +519,90 @@ saveatt (void *item, void *data)
 }
 
 static int
-add_attachments (mu_message_t *pmsg, mu_mime_t *pmime)
+add_body (mu_message_t inmsg, compose_env_t *env)
 {
-  mu_message_t inmsg, outmsg, part;
+  int rc;
   mu_body_t body;
+  mu_stream_t str;
+  struct atchinfo *aptr;
+  
+  mu_message_get_body (inmsg, &body);
+  mu_body_get_streamref (body, &str);
+
+  aptr = mu_alloc (sizeof (*aptr));
+  aptr->id = NULL;
+  aptr->encoding = default_encoding ? mu_strdup (default_encoding) : NULL;
+  attach_set_content_type (aptr, default_content_type);
+  aptr->name = NULL;
+  aptr->filename = NULL;
+  aptr->source = str;
+  if (str)
+    mu_stream_ref (str);
+  aptr->skip_empty = skip_empty_attachments || multipart_alternative;
+  if (!env->attlist)
+    env->attlist = attlist_new ();
+  rc = mu_list_prepend (env->attlist, aptr);
+  if (rc)
+    mu_diag_funcall (MU_DIAG_ERROR, "mu_list_prepend", NULL, rc);
+  return rc;
+}
+  
+static int
+add_attachments (compose_env_t *env, mu_message_t *pmsg)
+{
+  mu_message_t inmsg, outmsg;
   mu_header_t inhdr, outhdr;
   mu_iterator_t itr;
-  mu_mime_t mime;
-  mu_stream_t str, output;
   int rc;
-  char *p;
-  
-  if (mu_list_is_empty (attlist))
-    {
-      *pmime = NULL;
-      return 0;
-    }
   
   inmsg = *pmsg;
 
+  if (mailvar_is_true ("mime") && add_body (inmsg, env))
+    return 1;
+  
+  if (mu_list_is_empty (env->attlist))
+    return 0;
+  
   /* Create a mime object */
-  rc = mu_mime_create (&mime, NULL, 0);
+  rc = mu_mime_create (&env->mime, NULL,
+		       env->alt ?
+		         MU_MIME_MULTIPART_ALT : MU_MIME_MULTIPART_MIXED);
   if (rc)
     {
       mu_diag_funcall (MU_DIAG_ERROR, "mu_mime_create", NULL, rc);
       return 1;
     }
 
-  /* Add original message as its first part */
-  /* 1. Create the part and obtain a reference to its stream */
-  mu_message_create (&part, NULL);
-  mu_message_get_body (part, &body);
-  mu_body_get_streamref (body, &output);
-
-  /* 2. Get original body stream and copy it out to the part's body */
-  mu_message_get_body (inmsg, &body);
-  mu_body_get_streamref (body, &str);
-  mu_stream_copy (output, str, 0, NULL);
-
-  mu_stream_close (output);
-  mu_stream_destroy (&output);
-
   mu_message_get_header (inmsg, &inhdr);
-  mu_header_get_iterator (inhdr, &itr);
-
-  /* 3. Copy "Content-*" headers from the original message */
-  mu_message_get_header (part, &outhdr);
-  for (mu_iterator_first (itr); !mu_iterator_is_done (itr);
-       mu_iterator_next (itr))
-    {
-      const char *name, *value;
-      
-      if (mu_iterator_current_kv (itr, (const void **)&name,
-				  (void**)&value) == 0)
-	{
-	  if (mu_c_strncasecmp (name, "Content-", 8) == 0)
-	    mu_header_set_value (outhdr, name, value, 0);
-	}
-    }
-  
-  /* 4. Add the content type and content ID headers. */
-  mu_header_set_value (outhdr, MU_HEADER_CONTENT_TYPE, "text/plain", 0);
-  mu_rfc2822_msg_id (0, &p);
-  mu_header_set_value (outhdr, MU_HEADER_CONTENT_ID, p, 1);
-  free (p);
-
-  /* 5. Add part to the mime object */
-  mu_mime_add_part (mime, part);
-  mu_message_unref (part);
 
   /* Add the respective attachments */
-  rc = mu_list_foreach (attlist, saveatt, mime);
+  rc = mu_list_foreach (env->attlist, saveatt, env);
   if (rc)
-    {
-      mu_mime_destroy (&mime);
-      return 1;
-    }
+    return 1;
 
   /* Get the resulting message */
-  rc = mu_mime_get_message (mime, &outmsg);
+  rc = mu_mime_get_message (env->mime, &outmsg);
 
   if (rc)
     {
       mu_diag_funcall (MU_DIAG_ERROR, "mu_mime_get_message", NULL, rc);
-      mu_mime_destroy (&mime);
       return 1;
     }
 
   /* Copy rest of headers from the original message */
-  mu_message_get_header (outmsg, &outhdr);
+  rc = mu_message_get_header (outmsg, &outhdr);
+  if (rc)
+    {
+      mu_diag_funcall (MU_DIAG_ERROR, "mu_message_get_header", NULL, rc);
+      return 1;
+    }
+
+  rc = mu_header_get_iterator (inhdr, &itr);
+  if (rc)
+    {
+      mu_diag_funcall (MU_DIAG_ERROR, "mu_header_get_iterator", NULL, rc);
+      return 1;
+    }
   for (mu_iterator_first (itr); !mu_iterator_is_done (itr);
        mu_iterator_next (itr))
     {
@@ -434,18 +611,25 @@ add_attachments (mu_message_t *pmsg, mu_mime_t *pmime)
       if (mu_iterator_current_kv (itr, (const void **)&name,
 				  (void**)&value) == 0)
 	{
-	  if (mu_c_strcasecmp (name, MU_HEADER_MIME_VERSION) == 0 ||
-	      mu_c_strncasecmp (name, "Content-", 8) == 0)
-	    continue;
-	  mu_header_append (outhdr, name, value);
+	  if (mu_c_strcasecmp (name, MU_HEADER_RECEIVED) == 0
+	      || mu_c_strncasecmp (name, "X-", 2) == 0)
+	    mu_header_append (outhdr, name, value);
+	  else if (mu_c_strcasecmp (name, MU_HEADER_MIME_VERSION) == 0 ||
+		   mu_c_strncasecmp (name, "Content-", 8) == 0)
+	    {
+	      mu_error (_("%s: not setting header"), name);
+	      continue;
+	    }
+	  else
+	    mu_header_set_value (outhdr, name, value, 1);
 	}
     }
   mu_iterator_destroy (&itr);
 
   mu_message_unref (outmsg);
   mu_message_unref (inmsg);
+
   *pmsg = outmsg;
-  *pmime = mime;
   return 0;
 }
 
@@ -456,10 +640,10 @@ add_attachments (mu_message_t *pmsg, mu_mime_t *pmime)
 static void
 read_cc_bcc (compose_env_t *env)
 {
-  if (mailvar_get (NULL, "askcc", mailvar_type_boolean, 0) == 0)
+  if (mailvar_is_true ("askcc"))
     compose_header_set (env, MU_HEADER_CC,
 			ml_readline_with_intr ("Cc: "), COMPOSE_REPLACE);
-  if (mailvar_get (NULL, "askbcc", mailvar_type_boolean, 0) == 0)
+  if (mailvar_is_true ("askbcc"))
     compose_header_set (env, MU_HEADER_BCC,
 			ml_readline_with_intr ("Bcc: "), COMPOSE_REPLACE);
 }
@@ -492,7 +676,7 @@ mail_send (int argc, char **argv)
       if (interactive)
 	compose_header_set (&env, MU_HEADER_TO, ml_readline_with_intr ("To: "),
 			    COMPOSE_REPLACE);
-      else if (!mailvar_get (NULL, "editheaders", mailvar_type_boolean, 0))
+      else if (mailvar_is_true ("editheaders"))
 	{
 	  if (parse_headers (mu_strin, &env) != parse_headers_ok)
 	    {
@@ -547,10 +731,10 @@ mail_send (int argc, char **argv)
 
   if (interactive)
     {
-      if (mailvar_get (NULL, "mailx", mailvar_type_boolean, 0))
+      if (!mailvar_is_true ("mailx"))
 	read_cc_bcc (&env);
 
-      if (mailvar_get (NULL, "asksub", mailvar_type_boolean, 0) == 0)
+      if (mailvar_is_true ("asksub"))
 	compose_header_set (&env, MU_HEADER_SUBJECT,
 			    ml_readline_with_intr ("Subject: "),
 			    COMPOSE_REPLACE);
@@ -620,7 +804,7 @@ parse_headers (mu_stream_t input, compose_env_t *env)
 	      
 	      if (name)
 		{
-		  mu_header_set_value (header, name, value[0] ? value : NULL, 0);
+		  mu_header_append (header, name, value[0] ? value : NULL);
 		  free (name);
 		  free (value);
 		  name = value = NULL;
@@ -650,7 +834,7 @@ parse_headers (mu_stream_t input, compose_env_t *env)
   free (buf);
   if (name)
     {
-      mu_header_set_value (header, name, value, 0);
+      mu_header_append (header, name, value);
       free (name);
       free (value);
     }     
@@ -671,6 +855,8 @@ void
 compose_init (compose_env_t *env)
 {
   memset (env, 0, sizeof (*env));
+  env->alt = multipart_alternative;
+  env->attlist = attlist_copy (attachment_list);
   mu_list_foreach (add_header_list, seed_headers, env);
 }
 
@@ -694,25 +880,32 @@ compose_header_set (compose_env_t *env, const char *name,
   switch (mode)
     {
     case COMPOSE_REPLACE:
-    case COMPOSE_APPEND:
-      if (is_address_field (name)
-	  && mailvar_get (NULL, "inplacealiases", mailvar_type_boolean, 0) == 0)
+      if (is_address_field (name) && mailvar_is_true ("inplacealiases"))
 	{
 	  char *exp = alias_expand (value);
-	  status = mu_header_set_value (env->header, name, exp ? exp : value,
-					mode);
+	  status = mu_header_set_value (env->header, name, exp ? exp : value, 1);
 	  free (exp);
 	}
       else
-	status = mu_header_set_value (env->header, name, value, mode);
+	status = mu_header_set_value (env->header, name, value, 1);
       break;
 
+    case COMPOSE_APPEND:
+      if (is_address_field (name) && mailvar_is_true ("inplacealiases"))
+	{
+	  char *exp = alias_expand (value);
+	  status = mu_header_append (env->header, name, exp ? exp : value);
+	  free (exp);
+	}
+      else
+	status = mu_header_append (env->header, name, value);
+      break;
+      
     case COMPOSE_SINGLE_LINE:
       if (mu_header_aget_value (env->header, name, &old_value) == 0
 	  && old_value[0])
 	{
-	  if (is_address_field (name)
-	      && mailvar_get (NULL, "inplacealiases", mailvar_type_boolean, 0) == 0)
+	  if (is_address_field (name) && mailvar_is_true ("inplacealiases"))
 	    {
 	      char *exp = alias_expand (value);
 	      status = util_merge_addresses (&old_value, exp ? exp : value);
@@ -759,6 +952,9 @@ compose_destroy (compose_env_t *env)
 {
   mu_header_destroy (&env->header);
   free (env->outfiles);
+  mu_mime_destroy (&env->mime);
+  mu_list_destroy (&env->attlist);
+  mu_stream_destroy (&env->compstr);
 }
 
 static int
@@ -793,7 +989,7 @@ fill_body (mu_message_t msg, mu_stream_t instr)
   
   if (n == 0)
     {
-      if (mailvar_get (NULL, "nullbody", mailvar_type_boolean, 0) == 0)
+      if (mailvar_is_true ("nullbody"))
 	{
 	  char *str;
 	  if (mailvar_get (&str, "nullbodymsg", mailvar_type_string, 0) == 0)
@@ -809,7 +1005,7 @@ fill_body (mu_message_t msg, mu_stream_t instr)
 static int
 save_dead_message_env (compose_env_t *env)
 {
-  if (mailvar_get (NULL, "save", mailvar_type_boolean, 0) == 0)
+  if (mailvar_is_true ("save"))
     {
       mu_stream_t dead_letter, str;
       int rc;
@@ -827,8 +1023,7 @@ save_dead_message_env (compose_env_t *env)
 	  mu_error (_("Cannot open file %s: %s"), name, strerror (rc));
 	  return 1;
 	}
-      if (mailvar_get (NULL, "appenddeadletter",
-		       mailvar_type_boolean, 0) == 0)
+      if (mailvar_is_true ("appenddeadletter"))
 	mu_stream_seek (dead_letter, 0, MU_SEEK_END, NULL);
       else
 	mu_stream_truncate (dead_letter, 0);
@@ -861,7 +1056,7 @@ save_dead_message_env (compose_env_t *env)
 static int
 save_dead_message (mu_message_t msg)
 {
-  if (mailvar_get (NULL, "save", mailvar_type_boolean, 0) == 0)
+  if (mailvar_is_true ("save"))
     {
       mu_stream_t dead_letter, str;
       int rc;
@@ -879,8 +1074,7 @@ save_dead_message (mu_message_t msg)
 	  mu_error (_("Cannot open file %s: %s"), name, strerror (rc));
 	  return 1;
 	}
-      if (mailvar_get (NULL, "appenddeadletter",
-		       mailvar_type_boolean, 0) == 0)
+      if (mailvar_is_true ("appenddeadletter"))
 	mu_stream_seek (dead_letter, 0, MU_SEEK_END, NULL);
       else
 	mu_stream_truncate (dead_letter, 0);
@@ -939,7 +1133,7 @@ send_message (mu_message_t msg)
 		    }
 		} 
 
-	      if (mailvar_get (NULL, "verbose", mailvar_type_boolean, 0) == 0)
+	      if (mailvar_is_true ("verbose"))
 		{
 		  mu_debug_set_category_level (MU_DEBCAT_MAILER,
 					  MU_DEBUG_LEVEL_UPTO (MU_DEBUG_PROT));
@@ -998,7 +1192,6 @@ mail_send0 (compose_env_t *env, int save_to)
   if (rc)
     {
       mu_error (_("Cannot open temporary file: %s"), mu_strerror (rc));
-      mu_list_destroy (&attlist);
       return 1;
     }
 
@@ -1011,7 +1204,7 @@ mail_send0 (compose_env_t *env, int save_to)
 
       if (ml_got_interrupt ())
 	{
-	  if (mailvar_get (NULL, "ignore", mailvar_type_boolean, 0) == 0)
+	  if (mailvar_is_true ("ignore"))
 	    {
 	      mu_printf ("@\n");
 	    }
@@ -1028,12 +1221,11 @@ mail_send0 (compose_env_t *env, int save_to)
 
       if (!buf)
 	{
-	  if (interactive 
-	      && mailvar_get (NULL, "ignoreeof", mailvar_type_boolean, 0) == 0)
+	  if (interactive && mailvar_is_true ("ignoreeof"))
 	    {
-	      mu_error (mailvar_get (NULL, "dot", mailvar_type_boolean, 0) == 0 ?
-			  _("Use \".\" to terminate letter.") :
-			  _("Use \"~.\" to terminate letter."));
+	      mu_error (mailvar_is_true ("dot") 
+                          ?  _("Use \".\" to terminate letter.") 
+                          : _("Use \"~.\" to terminate letter."));
 	      continue;
 	    }
 	  else
@@ -1042,8 +1234,7 @@ mail_send0 (compose_env_t *env, int save_to)
 
       int_cnt = 0;
 
-      if (strcmp (buf, ".") == 0
-	  && mailvar_get (NULL, "dot", mailvar_type_boolean, 0) == 0)
+      if (strcmp (buf, ".") == 0 && mailvar_is_true ("dot"))
 	done = 1;
       else if (mailvar_get (&escape, "escape", mailvar_type_string, 0) == 0
 	       && buf[0] == escape[0])
@@ -1094,24 +1285,20 @@ mail_send0 (compose_env_t *env, int save_to)
   if (int_cnt)
     {
       save_dead_message_env (env);
-      mu_stream_destroy (&env->compstr);
-      mu_list_destroy (&attlist);
       return 1;
     }
 
   /* In mailx compatibility mode, ask for Cc and Bcc after editing
      the body of the message */
-  if (mailvar_get (NULL, "mailx", mailvar_type_boolean, 0) == 0)
+  if (mailvar_is_true ("mailx"))
     read_cc_bcc (env);
 
   /* Prepare the header */
-  if (mailvar_get (NULL, "xmailer", mailvar_type_boolean, 0) == 0)
-    mu_header_set_value (env->header, MU_HEADER_X_MAILER, 
-                         program_version, 1);
+  if (mailvar_is_true ("xmailer"))
+    mu_header_set_value (env->header, MU_HEADER_X_MAILER, program_version, 1);
 
   if (util_header_expand (&env->header) == 0)
     {
-      mu_mime_t mime = NULL;
       mu_message_t msg = NULL;
       int status = 0;
       int sendit = (compose_header_get (env, MU_HEADER_TO, NULL) ||
@@ -1122,6 +1309,7 @@ mail_send0 (compose_env_t *env, int save_to)
 	  status = mu_message_create (&msg, NULL);
 	  if (status)
 	    break;
+	  
 	  /* Fill the body. */
 	  mu_stream_seek (env->compstr, 0, MU_SEEK_SET, NULL);
 	  status = fill_body (msg, env->compstr);
@@ -1130,8 +1318,8 @@ mail_send0 (compose_env_t *env, int save_to)
 	  
 	  mu_message_set_header (msg, env->header, NULL);
 	  env->header = NULL;
-	  
-	  status = add_attachments (&msg, &mime);
+
+	  status = add_attachments (env, &msg);
 	  if (status)
 	    break;
 	  
@@ -1208,14 +1396,10 @@ mail_send0 (compose_env_t *env, int save_to)
 
       mu_stream_destroy (&env->compstr);
       mu_message_destroy (&msg, NULL);
-      mu_mime_destroy (&mime);
       return status;
     }
   else
     save_dead_message_env (env);
-  
-  mu_stream_destroy (&env->compstr);
-  mu_list_destroy (&attlist);
   return 1;
 }
 
