@@ -403,7 +403,7 @@ util_get_crt ()
 
   if (mailvar_get (&lines, "crt", mailvar_type_number, 0) == 0)
     return lines;
-  else if (mailvar_get (NULL, "crt", mailvar_type_boolean, 0) == 0)
+  else if (mailvar_is_true ("crt"))
     return util_getlines ();
   return 0;
 }
@@ -504,7 +504,7 @@ util_get_sender (int msgno, int strip)
   mu_message_t msg = NULL;
   mu_address_t addr = NULL;
   char *buf = NULL, *p;
-
+  
   mu_mailbox_get_message (mbox, msgno, &msg);
   addr = get_sender_address (msg);
   if (!addr)
@@ -520,7 +520,7 @@ util_get_sender (int msgno, int strip)
 	}
     }
 
-  if (mu_address_aget_email (addr, 1, &buf))
+  if (mu_address_aget_email (addr, 1, &buf) || !buf)
     {
       mu_error (_("Cannot determine sender name (msg %d)"), msgno);
       mu_address_destroy (&addr);
@@ -706,7 +706,7 @@ util_save_outgoing (mu_message_t msg, char *savefile)
       if (rc)
 	{
 	  mu_error (_("Cannot create output mailbox `%s': %s"),
-		      filename, strerror (rc));
+		      filename, mu_strerror (rc));
 	  free (filename);
 	  return;
 	}
@@ -714,13 +714,13 @@ util_save_outgoing (mu_message_t msg, char *savefile)
       rc = mu_mailbox_open (outbox, MU_STREAM_WRITE | MU_STREAM_CREAT);
       if (rc)
 	mu_error (_("Cannot open output mailbox `%s': %s"),
-		    filename, strerror (rc));
+		    filename, mu_strerror (rc));
       else
 	{
 	  rc = mu_mailbox_append_message (outbox, msg);
 	  if (rc)
 	    mu_error (_("Cannot append message to `%s': %s"),
-			filename, strerror (rc));
+			filename, mu_strerror (rc));
 	}
 
       mu_mailbox_close (outbox);
@@ -851,6 +851,8 @@ util_merge_addresses (char **addr_str, const char *value)
       rc = mu_address_aget_printable (addr, &val);
       if (rc == 0)
 	{
+	  if (!val)
+	    return MU_ERR_NOENT;
 	  free (*addr_str);
 	  *addr_str = val;
 	}
@@ -931,12 +933,11 @@ util_header_expand (mu_header_t *phdr)
 	      mu_address_t new_addr;
 	      char *p = ws.ws_wordv[j];
 	      
-	      /* If inplacealiases was set, the value was already expanded */
-	      if (mailvar_get (NULL, "inplacealiases",
-			       mailvar_type_boolean, 0))
-		exp = alias_expand (p);
-	      else
+	      if (mailvar_is_true ("inplacealiases"))
+	        /* If inplacealiases was set, the value was already expanded */
 		exp = p;
+	      else
+		exp = alias_expand (p);
 	      rc = mu_address_create (&new_addr, p);
 	      if (rc)
 		{
@@ -958,13 +959,13 @@ util_header_expand (mu_header_t *phdr)
 	      const char *newvalue;
 	      
 	      rc = mu_address_sget_printable (addr, &newvalue);
-	      if (rc == 0)
+	      if (rc == 0 && newvalue)
 		mu_header_set_value (hdr, name, newvalue, 1);
 	      mu_address_destroy (&addr);
 	    }
 	}
       else
-	mu_header_set_value (hdr, name, value, 0);
+	mu_header_append (hdr, name, value);
     }
 
   if (errcnt == 0)
@@ -1045,46 +1046,53 @@ util_run_cached_commands (mu_list_t *list)
   mu_list_destroy (list);
 }
 
-void
-util_rfc2047_decode (char **value)
+char *
+util_get_charset (void)
 {
-  char *charset = NULL;
-  char *tmp;
-  int rc;
+  char *charset;
 
-  if (!*value || mailvar_get (&charset, "charset", mailvar_type_string, 0))
-    return;
+  if (mailvar_get (&charset, "charset", mailvar_type_string, 0))
+    return NULL;
 
   if (mu_c_strcasecmp (charset, "auto") == 0)
     {
-      static char *saved_charset;
-
-      if (!saved_charset)
+      struct mu_lc_all lc_all = { .flags = 0 };
+      char *tmp = getenv ("LC_ALL");
+      if (!tmp)
+	tmp = getenv ("LANG");
+      
+      if (tmp && mu_parse_lc_all (tmp, &lc_all, MU_LC_CSET) == 0)
 	{
-	  /* Try to deduce the charset from LC_ALL or LANG variables */
-
-	  tmp = getenv ("LC_ALL");
-	  if (!tmp)
-	    tmp = getenv ("LANG");
-
-	  if (tmp)
-	    {
-	      struct mu_lc_all lc_all;
-	      
-	      if (mu_parse_lc_all (tmp, &lc_all, MU_LC_CSET) == 0)
-		saved_charset = lc_all.charset;
-	    }
+	  charset = mu_strdup (lc_all.charset);
+	  mu_lc_all_free (&lc_all);
 	}
-      charset = saved_charset; /* NOTE: a minor memory leak */
+      else
+	charset = NULL;
     }
+  else
+    charset = mu_strdup (charset);
 
+  return charset;
+}
+
+void
+util_rfc2047_decode (char **value)
+{
+  char *charset, *tmp;
+  int rc;
+
+  if (!*value)
+    return;
+  charset = util_get_charset ();
   if (!charset)
     return;
-  
+
   rc = mu_rfc2047_decode (charset, *value, &tmp);
+  free (charset);
+  
   if (rc)
     {
-      if (mailvar_get (NULL, "verbose", mailvar_type_boolean, 0) == 0)
+      if (mailvar_is_true ("verbose"))
 	mu_error (_("Cannot decode line `%s': %s"), *value, mu_strerror (rc));
     }
   else
