@@ -377,17 +377,19 @@ finish_draft ()
   mu_stream_destroy (&stream);
 }
 
+static struct mh_optinit optinit[] = {
+  { "draftfolder", "Draft-Folder" },
+  { "whatnowproc", "whatnowproc" },
+  { NULL }
+};
+
 int
 main (int argc, char **argv)
 {
   int rc;
 
-  mh_getopt (&argc, &argv, options, MH_GETOPT_DEFAULT_FOLDER,
-	     args_doc, prog_doc, NULL);
-  if (!draftfolder)
-    draftfolder = mh_global_profile_get ("Draft-Folder", NULL);
-  if (!whatnowproc)
-    whatnowproc = mh_global_profile_get ("whatnowproc", NULL);
+  mh_getopt_ext (&argc, &argv, options, MH_GETOPT_DEFAULT_FOLDER, optinit,
+		 args_doc, prog_doc, NULL);
   if (!formfile)
     mh_find_file ("forwcomps", &formfile);
   
