@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003, 2005-2012, 2014-2015 Free Software Foundation,
+   Copyright (C) 2003, 2005-2012, 2014-2017 Free Software Foundation,
    Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
@@ -22,99 +22,9 @@
 #include <mailutils/mime.h>
 #include <setjmp.h>
 
-static char doc[] = N_("GNU MH mhn")"\v"
-N_("Options marked with `*' are not yet implemented.\n\
-Use -help to obtain the list of traditional MH options.");
+static char prog_doc[] = N_("manipulate MIME messages");
 static char args_doc[] = N_("[MSGLIST]");
 
-static struct argp_option options[] = {
-  {"folder",       ARG_FOLDER,       N_("FOLDER"), 0,
-   N_("specify folder to operate upon"), 0},
-  {"file",         ARG_FILE,         N_("FILE"),   0,
-   N_("specify file to operate upon"), 0},
-
-#define GRID 10
-  {N_("MIME editing options"),   0,  NULL, OPTION_DOC,  NULL, GRID},
-  {"compose",      ARG_COMPOSE,      N_("BOOL"), OPTION_ARG_OPTIONAL,
-   N_("compose the MIME message (default)"), GRID+1},
-  {"build",        0,                NULL,       OPTION_ALIAS,
-   NULL, GRID+1 },
-  {"nocompose",    ARG_NOCOMPOSE,    NULL, OPTION_HIDDEN, NULL, GRID+1},
-  {"nobuild",      ARG_NOCOMPOSE,    NULL, OPTION_HIDDEN|OPTION_ALIAS,
-   NULL, GRID+1},
-#undef GRID
-#define GRID 20  
-  {N_("Listing options"),   0,  NULL, OPTION_DOC,  NULL, GRID},
-  {"list",         ARG_LIST,         N_("BOOL"), OPTION_ARG_OPTIONAL,
-   N_("list the table of contents"), GRID+1 },
-  {"nolist",       ARG_NOLIST,       NULL, OPTION_HIDDEN, "", GRID+1 },
-  {"headers",      ARG_HEADER,       N_("BOOL"), OPTION_ARG_OPTIONAL,
-   N_("print the banner above the listing"), GRID+1 },
-  {"noheaders",    ARG_NOHEADERS,    NULL, OPTION_HIDDEN, "", GRID+1 },
-  {"realsize",     ARG_REALSIZE,     N_("BOOL"), OPTION_ARG_OPTIONAL,
-   N_("list the decoded sizes"), GRID+1 },
-  {"norealsize",   ARG_NOREALSIZE,   NULL, OPTION_HIDDEN, "", GRID+1 },
-#undef GRID
-#define GRID 40
-  {N_("Display options"),   0,  NULL, OPTION_DOC,  NULL, GRID},
-  {"show",         ARG_SHOW,         N_("BOOL"), OPTION_ARG_OPTIONAL,
-   N_("display the contents of the messages"), GRID+1},
-  {"noshow",       ARG_NOSHOW,       NULL, OPTION_HIDDEN, "", GRID+1 },
-  {"serialonly",   ARG_SERIALONLY,   N_("BOOL"), OPTION_ARG_OPTIONAL,
-   N_("* display messages serially"), GRID+1},
-  {"noserialonly", ARG_NOSERIALONLY, NULL, OPTION_HIDDEN, "", GRID+1 },
-  {"form",         ARG_FORM,         N_("FILE"), 0,
-   N_("read mhl format from FILE"), GRID+1},
-  {"pause",        ARG_PAUSE,        N_("BOOL"), OPTION_ARG_OPTIONAL,
-   N_("pause prior to displaying content"), GRID+1},
-  {"nopause",      ARG_NOPAUSE,      NULL, OPTION_HIDDEN, "", GRID+1 },
-#undef GRID
-#define GRID 50
-  {N_("Saving options"),   0,  NULL, OPTION_DOC,  NULL, GRID},
-  {"store",        ARG_STORE,        N_("BOOL"), OPTION_ARG_OPTIONAL,
-   N_("write extracted message parts to disk"), GRID+1},
-  {"nostore",      ARG_NOSTORE,      NULL, OPTION_HIDDEN, "", GRID+1 },
-  {"auto",         ARG_AUTO,         N_("BOOL"), OPTION_ARG_OPTIONAL,
-   N_("use filenames from the content headers"), GRID+1},
-  {"noauto",       ARG_NOAUTO,       NULL, OPTION_HIDDEN, "", GRID+1 },
-  {"charset",      ARG_CHARSET,      N_("NAME"), 0,
-   N_("use this charset to represent attachment file names"), GRID+1},
-#undef GRID
-#define GRID 60
-  {N_("Other options"),    0,  NULL, OPTION_DOC,  NULL, GRID},
-  {"part",         ARG_PART,         N_("PART"), 0,
-   N_("limit the scope of the operation to the given part"), GRID+1},
-  {"type",         ARG_TYPE,         N_("CONTENT"), 0,
-   N_("operate on message part with given multipart content"), GRID+1 },
-  {"verbose",      ARG_VERBOSE,     N_("BOOL"), OPTION_ARG_OPTIONAL,
-   N_("print additional information"), GRID+1 },
-  {"noverbose",    ARG_NOVERBOSE,   NULL, OPTION_HIDDEN, "", GRID+1 },
-  {"quiet",        ARG_QUIET, 0, 0,
-   N_("be quiet"), GRID+1},
-#undef GRID
-  {NULL}
-};
-
-/* Traditional MH options */
-struct mh_option mh_option[] = {
-  { "file",          MH_OPT_ARG, "filename" },
-  { "compose" },
-  { "build" },
-  { "list",          MH_OPT_BOOL },
-  { "headers",       MH_OPT_BOOL },
-  { "realsize",      MH_OPT_BOOL },
-  { "show",          MH_OPT_BOOL },
-  { "serialonly",    MH_OPT_BOOL },
-  { "form",          MH_OPT_ARG, "formfile" },
-  { "pause",         MH_OPT_BOOL },
-  { "store",         MH_OPT_BOOL },
-  { "auto",          MH_OPT_BOOL },
-  { "part",          MH_OPT_ARG, "part" },
-  { "type",          MH_OPT_ARG, "type" },
-  { "verbose",       MH_OPT_BOOL },
-  { NULL }
-};
-
 typedef struct _msg_part *msg_part_t;
 
 static msg_part_t msg_part_create (size_t num);
@@ -124,28 +34,22 @@ static void msg_part_incr (msg_part_t p);
 static void msg_part_decr (msg_part_t p);
 static void msg_part_set_subpart (msg_part_t p, size_t subpart);
 static void msg_part_print (msg_part_t p, int width);
-static msg_part_t msg_part_parse (char *str);
+static msg_part_t msg_part_parse (char const *str);
 static int msg_part_level (msg_part_t p);
 static size_t msg_part_subpart (msg_part_t p, int level);
 
-enum mhn_mode
-  {
-    mode_compose,
-    mode_list,
-    mode_show,
-    mode_store,
-  };
+static int compose_option;
+static int list_option; 
+static int show_option;
+static int store_option;
 
-static enum mhn_mode mode = mode_compose;
+static int headers_option = 1;
+static int realsize_option;
+static int auto_option;
+/*static int serialonly_option;*/
+static int verbose_option;
+static int quiet_option;
 
-#define OPT_HEADERS    001
-#define OPT_REALSIZE   002
-#define OPT_AUTO       004
-#define OPT_SERIALONLY 010
-#define OPT_VERBOSE    020
-#define OPT_QUIET      040
-
-static int mode_options = OPT_HEADERS;
 static int pause_option = -1; /* -pause option. -1 means "not given" */
 static char *formfile;
 static char *content_type;
@@ -199,7 +103,7 @@ split_content (const char *content, char **type, char **subtype)
 }
 
 static void
-split_args (const char *argstr, size_t len, int *pargc, char ***pargv)
+split_args (const char *argstr, size_t len, size_t *pargc, char ***pargv)
 {
   struct mu_wordsplit ws;
 
@@ -215,10 +119,7 @@ split_args (const char *argstr, size_t len, int *pargc, char ***pargv)
     }
   else
     {
-      *pargc = ws.ws_wordc;
-      *pargv = ws.ws_wordv;
-      ws.ws_wordc = 0;
-      ws.ws_wordv = NULL;
+      mu_wordsplit_get_words (&ws, pargc, pargv);
       mu_wordsplit_free (&ws);
     }
 }
@@ -263,173 +164,88 @@ _get_content_encoding (mu_header_t hdr, char **value)
   *value = encoding;
   return 0;
 }
-
-static error_t
-opt_handler (int key, char *arg, struct argp_state *state)
+
+static void
+set_part (struct mu_parseopt *po, struct mu_option *opt, char const *arg)
 {
-  switch (key)
-    {
-    case ARG_FOLDER: 
-      mh_set_current_folder (arg);
-      break;
-
-    case ARG_FILE:
-      input_file = arg;
-      break;
-      
-      /* Operation mode */
-    case ARG_COMPOSE:
-      if (is_true (arg))
-	{
-	  mode = mode_compose;
-	  break;
-	}
-      /*FALLTHRU*/
-    case ARG_NOCOMPOSE:
-      if (mode == mode_compose)
-	mode = mode_compose;
-      break;
-
-    case ARG_LIST:
-      if (is_true (arg))
-	{
-	  mode = mode_list;
-	  break;
-	}
-      /*FALLTHRU*/
-    case ARG_NOLIST:
-      if (mode == mode_list)
-	mode = mode_compose;
-      break;
-
-    case ARG_SHOW:
-      if (is_true (arg))
-	{
-	  mode = mode_show;
-	  break;
-	}
-      /*FALLTHRU*/
-    case ARG_NOSHOW:
-      if (mode == mode_show)
-	mode = mode_compose;
-      break;
-
-    case ARG_STORE:
-      if (is_true (arg))
-	{
-	  mode = mode_store;
-	  break;
-	}
-      /*FALLTHRU*/
-    case ARG_NOSTORE:
-      if (mode == mode_store)
-	mode = mode_compose;
-      break;
-
-      /* List options */
-    case ARG_HEADER:
-      if (is_true (arg))
-	{
-	  mode_options |= OPT_HEADERS;
-	  break;
-	}
-      /*FALLTHRU*/
-    case ARG_NOHEADERS:
-      mode_options &= ~OPT_HEADERS;
-      break;
-      
-    case ARG_REALSIZE:
-      if (is_true (arg))
-	{
-	  mode_options |= OPT_REALSIZE;
-	  break;
-	}
-      /*FALLTHRU*/
-    case ARG_NOREALSIZE:
-      mode_options &= ~OPT_REALSIZE;
-      break;
-
-      /* Display options */
-
-    case ARG_SERIALONLY:
-      mh_opt_notimpl_warning ("-[no]serialonly");
-      if (is_true (arg))
-	{
-	  mode_options |= OPT_SERIALONLY;
-	  break;
-	}
-      /*FALLTHRU*/
-    case ARG_NOSERIALONLY:
-      mode_options &= ~OPT_SERIALONLY;
-      break;
-
-    case ARG_PAUSE:
-      pause_option = is_true (arg);
-      break;
-      
-    case ARG_NOPAUSE:
-      pause_option = 0;
-      break;
-
-      /* Store options */
-    case ARG_AUTO:
-      if (is_true (arg))
-	{
-	  mode_options |= OPT_AUTO;
-	  break;
-	}
-      /*FALLTHRU*/
-    case ARG_NOAUTO:
-      mode_options &= ~OPT_AUTO;
-      break;
-
-    case ARG_FORM:
-      mh_find_file (arg, &formfile);
-      break;
-
-      /* Common options */
-    case ARG_VERBOSE:
-      if (is_true (arg))
-	{
-	  mode_options |= OPT_VERBOSE;
-	  break;
-	}
-      /*FALLTHRU*/
-    case ARG_NOVERBOSE:
-      mode_options &= ~OPT_VERBOSE;
-      break;
-
-    case ARG_TYPE:
-      sfree (&content_type);
-      sfree (&content_subtype);
-      split_content (arg, &content_type, &content_subtype);
-      break;
-      
-    case ARG_PART:
-      req_part = msg_part_parse (arg);
-      break;
-
-    case ARG_QUIET:
-      mode_options |= OPT_QUIET;
-      break;
-	
-    case ARG_CHARSET:
-      charset = arg;
-      break;
-      
-    case ARGP_KEY_FINI:
-      if (!formfile)
-	mh_find_file ("mhl.headers", &formfile);
-      if (!isatty (0))
-	pause_option = 0;
-      break;
-      
-    default:
-      return ARGP_ERR_UNKNOWN;
-    }
-  return 0;
+  req_part = msg_part_parse (arg);
 }
 
+static void
+set_type (struct mu_parseopt *po, struct mu_option *opt, char const *arg)
+{
+  sfree (&content_type);
+  sfree (&content_subtype);
+  split_content (arg, &content_type, &content_subtype);
+}
+
+static struct mu_option options[] = {
+  { "folder",      0,       N_("FOLDER"), MU_OPTION_DEFAULT,
+    N_("specify folder to operate upon"),
+    mu_c_string, NULL, mh_opt_set_folder },
+  { "file",        0,       N_("FILE"),   MU_OPTION_DEFAULT,
+    N_("specify file to operate upon"),
+    mu_c_string, &input_file },
+
+  MU_OPTION_GROUP (N_("MIME editing options")),
+  { "compose",     0,       NULL, MU_OPTION_DEFAULT,
+    N_("compose the MIME message (default)"),
+    mu_c_bool, &compose_option },
+  { "build",       0,       NULL, MU_OPTION_ALIAS },
+
+  MU_OPTION_GROUP (N_("Listing options")),
+  { "list",        0,       NULL, MU_OPTION_DEFAULT,
+    N_("list the table of contents"),
+    mu_c_bool, &list_option },
+  { "headers",     0,       NULL, MU_OPTION_DEFAULT,
+    N_("print the banner above the listing"),
+    mu_c_bool, &headers_option },
+  { "realsize",    0,       NULL, MU_OPTION_DEFAULT,
+    N_("list the decoded sizes"),
+    mu_c_bool, &realsize_option },
+
+  MU_OPTION_GROUP (N_("Display options")),
+  { "show",        0,       NULL, MU_OPTION_DEFAULT,
+    N_("display the contents of the messages"),
+    mu_c_bool, &show_option },
+  { "serialonly",  0,       NULL, MU_OPTION_HIDDEN,
+    "",
+    mu_c_bool, NULL, mh_opt_notimpl_warning },
+  { "form",        0,       N_("FILE"), MU_OPTION_DEFAULT,
+    N_("read mhl format from FILE"),
+    mu_c_string, &formfile, mh_opt_find_file },
+  { "pause",       0,       NULL, MU_OPTION_DEFAULT,
+    N_("pause prior to displaying content"),
+    mu_c_bool, &pause_option },
+
+  MU_OPTION_GROUP (N_("Saving options")),
+  { "store",        0,       NULL, MU_OPTION_DEFAULT,
+    N_("write extracted message parts to disk"),
+    mu_c_bool, &store_option },
+  { "auto",         0,       NULL, MU_OPTION_DEFAULT,
+    N_("use filenames from the content headers"),
+    mu_c_bool, &auto_option },
+  { "charset",      0,       N_("NAME"), MU_OPTION_DEFAULT,
+    N_("use this charset to represent attachment file names"),
+    mu_c_string, &charset },
+
+  MU_OPTION_GROUP (N_("Other options")),
+  { "part",         0,       N_("PART"), MU_OPTION_DEFAULT,
+    N_("limit the scope of the operation to the given part"),
+    mu_c_string, NULL, set_part },
+  { "type",         0,       N_("CONTENT"), MU_OPTION_DEFAULT,
+    N_("operate on message part with given multipart content"),
+    mu_c_string, NULL, set_type },
+  { "verbose",      0,       NULL, MU_OPTION_DEFAULT,
+    N_("print additional information"),
+    mu_c_bool, &verbose_option },
+  { "quiet",        0,       NULL, MU_OPTION_DEFAULT,
+    N_("be quiet"),
+    mu_c_bool, &quiet_option },
+
+  MU_OPTION_END
+};
+
 
 /* *********************** Message part functions ************************* */
 
@@ -558,7 +374,7 @@ msg_part_format_pool (mu_opool_t pool, msg_part_t p)
 }
 
 msg_part_t
-msg_part_parse (char *str)
+msg_part_parse (char const *str)
 {
   msg_part_t p = msg_part_create (0);
 
@@ -631,7 +447,7 @@ mhn_compose_command (char *typestr, char *typeargs, int *flags, char *file)
 {
   const char *p, *str;
   char *type, *subtype, **typeargv = NULL;
-  int typeargc = 0;
+  size_t typeargc = 0;
   mu_opool_t pool;
 
   split_content (typestr, &type, &subtype);
@@ -645,7 +461,7 @@ mhn_compose_command (char *typestr, char *typeargs, int *flags, char *file)
     %F  %f, and stdout is not redirected
     %s  subtype */
 
-  mu_opool_create (&pool, 1);
+  mu_opool_create (&pool, MU_OPOOL_ENOMEMABRT);
 
   p = mu_str_skip_class (str, MU_CTYPE_SPACE);
   
@@ -662,7 +478,7 @@ mhn_compose_command (char *typestr, char *typeargs, int *flags, char *file)
 	      /* additional arguments */
 	      if (typeargs)
 		{
-		  int i;
+		  size_t i;
 		  
 		  if (!typeargv)
 		    split_args (typeargs, strlen (typeargs),
@@ -741,7 +557,7 @@ check_type (const char *typeargs, const char *typeval)
   
   if (typeargs)
     {
-      int i, argc;
+      size_t i, argc;
       char **argv;
 		      
       split_args (typeargs, strlen (typeargs), &argc, &argv);
@@ -769,7 +585,7 @@ mhn_show_command (mu_message_t msg, msg_part_t part, int *flags,
   mu_opool_t pool;
   mu_header_t hdr;
   char *temp_cmd = NULL;
-  int typeargc = 0;
+  size_t typeargc = 0;
   char **typeargv = NULL;
   
   mu_message_get_header (msg, &hdr);
@@ -826,7 +642,7 @@ mhn_show_command (mu_message_t msg, msg_part_t part, int *flags,
     %s  subtype
     %d  content description */
 
-  mu_opool_create (&pool, 1);
+  mu_opool_create (&pool, MU_OPOOL_ENOMEMABRT);
 
   p = mu_str_skip_class (str, MU_CTYPE_SPACE);
   
@@ -998,7 +814,7 @@ mhn_store_command (mu_message_t msg, msg_part_t part, const char *name,
      %p  part
      %s  subtype */
 
-  mu_opool_create (&pool, 1);
+  mu_opool_create (&pool, MU_OPOOL_ENOMEMABRT);
   
   for (p = str; *p; p++)
     {
@@ -1084,7 +900,7 @@ _message_is_external_body (mu_message_t msg, char ***env)
   rc = subtype && strcmp (subtype, "external-body") == 0;
   if (rc && env)
     {
-      int c;
+      size_t c;
       split_args (argstr, strlen (argstr), &c, env);
     }
 
@@ -1270,7 +1086,7 @@ mhn_message_size (mu_message_t msg, mu_off_t *psize)
   
   *psize = 0;
   mu_message_get_body (msg, &body);
-  if (mode_options & OPT_REALSIZE)
+  if (realsize_option)
     {
       mu_stream_t dstr = NULL, bstr = NULL;
 
@@ -1356,7 +1172,7 @@ list_handler (mu_message_t msg, msg_part_t part, char *type, char *encoding,
       get_extbody_params (msg, &content_type, &content_descr);
 	
       printf ("          ");
-      printf ("%-25s", content_type ? content_type : "");
+      printf ("%-25s", mu_prstr (content_type));
       if (content_descr)
 	printf ("       %s", content_descr);
       printf ("\n");
@@ -1388,11 +1204,11 @@ list_iterator (size_t num, mu_message_t msg, void *data)
 }
 
 int
-mhn_list ()
+mhn_list (void)
 {
   int rc; 
 
-  if (mode_options & OPT_HEADERS)
+  if (headers_option)
     printf (_(" msg part type/subtype              size  description\n"));
 
   if (message)
@@ -1513,7 +1329,7 @@ sigint (int sig)
 }
 
 static int
-mhn_pause ()
+mhn_pause (void)
 {
   char c;
   int rc = 0;
@@ -1622,7 +1438,7 @@ show_iterator (size_t num, mu_message_t msg, void *data)
 }
 
 int
-mhn_show ()
+mhn_show (void)
 {
   
   int rc;
@@ -1675,7 +1491,7 @@ store_handler (mu_message_t msg, msg_part_t part, char *type, char *encoding,
   if (mu_message_is_multipart (msg, &ismime) == 0 && ismime)
     return 0;
   
-  if (mode_options & OPT_AUTO)
+  if (auto_option)
     {
       char *val;
       int rc = mu_message_aget_decoded_attachment_name (msg, charset,
@@ -1776,7 +1592,7 @@ store_handler (mu_message_t msg, msg_part_t part, char *type, char *encoding,
       printf (_("storing message %s part %s as file %s\n"),
 	      prefix, partstr, name);
 
-      if (!(mode_options & OPT_QUIET) && access (name, R_OK) == 0)
+      if (!quiet_option && access (name, R_OK) == 0)
 	{
 	  int ok;
 	  
@@ -1864,7 +1680,7 @@ store_iterator (size_t num, mu_message_t msg, void *data)
 }
 
 int
-mhn_store ()
+mhn_store (void)
 {
   int rc = 0;
   
@@ -1928,7 +1744,7 @@ parse_brace (char **pval, char **cmd, int c, struct compose_env *env)
   return 0;
 }
 
-#define isdelim(c) (mu_isspace (c) || strchr (";<[(", c))
+#define isdelim(c) (c == 0 || mu_isspace (c) || strchr (";<[(", c))
 #define skipws(ptr) (ptr) = mu_str_skip_class (ptr, MU_CTYPE_SPACE)
 
 int
@@ -1938,7 +1754,7 @@ parse_content_type (struct compose_env *env,
   int status = 0, stop = 0;
   char *rest = *prest;
   char *comment = NULL;
-
+  
   while (stop == 0 && status == 0 && *rest)
     {
       skipws (rest);
@@ -2007,12 +1823,29 @@ parse_content_type (struct compose_env *env,
 	  rest++;
 	  mu_opool_append_char (pool, '=');
 	  skipws (rest);
-	  for (; *rest; rest++)
+	  if (*rest == '"')
 	    {
-	      if (isdelim (*rest))
-		break;
 	      mu_opool_append_char (pool, *rest);
+	      rest++;
+	      while (*rest != '"')
+		{
+		  if (*rest == '\\')
+		    {
+		      mu_opool_append_char (pool, *rest);
+		      if (rest[1])
+			rest++;
+		      else
+			break;
+		    }
+		  mu_opool_append_char (pool, *rest);
+		  rest++;
+		}
+	      mu_opool_append_char (pool, *rest);
+	      rest++;
 	    }
+	  else
+	    for (; !isdelim (*rest); rest++)
+	      mu_opool_append_char (pool, *rest);
 	  break;
 	    
 	default:
@@ -2051,7 +1884,7 @@ parse_type_command (char **pcmd, struct compose_env *env, mu_header_t hdr)
   char *rest = *pcmd;
   
   skipws (rest);
-  for (sp = rest; *sp && !isdelim (*sp); sp++)
+  for (sp = rest; !isdelim (*sp); sp++)
     ;
   c = *sp;
   *sp = 0;
@@ -2067,7 +1900,7 @@ parse_type_command (char **pcmd, struct compose_env *env, mu_header_t hdr)
       return 1;
     }
 
-  mu_opool_create (&pool, 1);
+  mu_opool_create (&pool, MU_OPOOL_ENOMEMABRT);
 
   mu_opool_appendz (pool, type);
   mu_opool_append_char (pool, '/');
@@ -2111,7 +1944,7 @@ copy_header (mu_message_t msg, mu_header_t out)
 	  mu_header_sget_field_value (hdr, i, &value))
 	continue;
 
-      mu_header_set_value (out, name, value, 0);
+      mu_header_append (out, name, value);
     }
 }
 
@@ -2176,7 +2009,7 @@ finish_text_msg (struct compose_env *env, mu_message_t *msg, int ascii)
       mu_message_get_header (newmsg, &hdr);
       copy_header (*msg, hdr);
       mu_header_set_value (hdr, MU_HEADER_CONTENT_TRANSFER_ENCODING,
-			   "quoted-printable", 0);
+			   "quoted-printable", 1);
       
       mu_message_get_body (newmsg, &body);
       mu_body_get_streamref (body, &output);
@@ -2234,7 +2067,7 @@ edit_extern (char *cmd, struct compose_env *env, mu_message_t *msg, int level)
 
   mu_message_get_header (*msg, &hdr);
 
-  mu_opool_create (&pool, 1);
+  mu_opool_create (&pool, MU_OPOOL_ENOMEMABRT);
   mu_opool_append (pool, EXTCONTENT, sizeof (EXTCONTENT) - 1);
   *--rest = ';'; /* FIXME */
   rc = parse_content_type (env, pool, &rest, &id, NULL);
@@ -2753,7 +2586,7 @@ parse_header_directive (const char *val, char **encoding, char **charset,
       p = strchr (val, '>');
       if (p)
 	{
-	  int i, argc;
+	  size_t i, argc;
 	  char **argv;
 	    
 	  *subject = mu_strdup (p + 1);
@@ -2819,7 +2652,7 @@ mhn_header (mu_message_t msg, mu_message_t omsg)
 		  _get_content_encoding (parthdr, &encoding);
 		  if (typeargs)
 		    {
-		      int i, argc;
+		      size_t i, argc;
 		      char **argv;
 		      
 		      split_args (typeargs, strlen (typeargs), &argc, &argv);
@@ -2864,7 +2697,7 @@ mhn_header (mu_message_t msg, mu_message_t omsg)
 }
 
 int
-mhn_compose ()
+mhn_compose (void)
 {
   int rc;
   mu_mime_t mime = NULL;
@@ -2927,12 +2760,22 @@ mhn_compose ()
   
   /* Preserve the backup copy and replace the draft */
   unlink (backup);
-  rename (input_file, backup);
-  rename (name, input_file);
+
+  rc = mu_rename_file (input_file, backup, MU_COPY_OVERWRITE);
+  if (rc)
+    mu_error (_("can't rename %s to backup file %s: %s"),
+	      input_file, backup, mu_strerror (rc));
+  else
+    {
+      rc = mu_rename_file (name, input_file, 0);
+      if (rc)
+	mu_error (_("can't rename %s to %s: %s"),
+		  name, input_file, mu_strerror (rc));
+    }
 
   free (name);
   mu_mime_unref (mime); 
-  return 0;
+  return rc;
 }
 
 
@@ -2942,17 +2785,16 @@ int
 main (int argc, char **argv)
 {
   int rc;
-  int index;
   
-  MU_APP_INIT_NLS ();
+  mh_getopt (&argc, &argv, options, 0, args_doc, prog_doc, NULL);
+  if (!formfile)
+    mh_find_file ("mhl.headers", &formfile);
+  if (!isatty (0))
+    pause_option = 0;
+
+  if (!compose_option && !list_option && !show_option && !store_option)
+    compose_option = 1;
   
-  mh_argp_init ();
-  mh_argp_parse (&argc, &argv, 0, options, mh_option, args_doc, doc,
-		 opt_handler, NULL, &index);
-
-  argc -= index;
-  argv += index;
-
   signal (SIGPIPE, SIG_IGN);
 
   if (input_file)
@@ -2969,45 +2811,37 @@ main (int argc, char **argv)
       if (!message)
 	return 1;
     }
-  else if (mode == mode_compose)
+  
+  if (compose_option)
     {
       if (argc > 1)
 	{
 	  mu_error (_("extra arguments"));
 	  return 1;
 	}
-      input_file = mh_expand_name (mu_folder_directory (),
-				   argc == 1 ? argv[0] : "draft", NAME_ANY);
-      message = mh_file_to_message (NULL, input_file);
-      if (!message)
-	return 1;
+      if (!input_file)
+	{
+	  input_file = mh_expand_name (mu_folder_directory (),
+				       argc == 1
+				         ? argv[0] : "draft", NAME_ANY);
+	  message = mh_file_to_message (NULL, input_file);
+	  if (!message)
+	    return 1;
+	}
+      rc = mhn_compose ();
     }
   else
     {
       mbox = mh_open_folder (mh_current_folder (), MU_STREAM_READ);
       mh_msgset_parse (&msgset, mbox, argc, argv, "cur");
-    }
-  
-  switch (mode)
-    {
-    case mode_compose:
-      rc = mhn_compose ();
-      break;
-      
-    case mode_list:
-      rc = mhn_list ();
-      break;
-      
-    case mode_show:
-      rc = mhn_show ();
-      break;
-      
-    case mode_store:
-      rc = mhn_store ();
-      break;
-      
-    default:
-      abort ();
+      /* FIXME: Combine the three */
+      rc = 0;
+      if (list_option)
+	rc |= mhn_list ();
+      if (show_option)
+	rc |= mhn_show ();
+      if (store_option)
+	rc |= mhn_store ();
     }
   return rc ? 1 : 0;
 }
