@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2003, 2005-2012, 2014-2015 Free Software
+   Copyright (C) 1999-2003, 2005-2012, 2014-2017 Free Software
    Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
@@ -26,115 +26,52 @@
 #include <time.h>
 #include <mailutils/observer.h>
 
-static char doc[] = N_("GNU MH scan")"\v"
-N_("Use -help to obtain the list of traditional MH options.");
-static char args_doc[] = N_("[+FOLDER] [MSGLIST]");
-
-/* GNU options */
-static struct argp_option options[] = {
-  {"folder",  ARG_FOLDER, N_("FOLDER"), 0,
-   N_("specify folder to scan")},
-  {"clear",   ARG_CLEAR, N_("BOOL"),   OPTION_ARG_OPTIONAL,
-   N_("clear screen after displaying the list")},
-  {"noclear", ARG_NOCLEAR, NULL, OPTION_HIDDEN, ""},
-  {"form",    ARG_FORM, N_("FILE"),   0,
-   N_("read format from given file")},
-  {"format",  ARG_FORMAT, N_("FORMAT"), 0,
-   N_("use this format string")},
-  {"header",  ARG_HEADER, N_("BOOL"),   OPTION_ARG_OPTIONAL,
-   N_("display header")},
-  {"width",   ARG_WIDTH, N_("NUMBER"), 0,
-   N_("set output width")},
-  {"reverse", ARG_REVERSE, N_("BOOL"),   OPTION_ARG_OPTIONAL,
-   N_("list messages in reverse order")},
-  {"noreverse", ARG_NOREVERSE, NULL, OPTION_HIDDEN, ""},
-  {"file",    ARG_FILE, N_("FILE"),   0,
-   N_("[not yet implemented]")},
-  
-  { 0 }
-};
-
-/* Traditional MH options */
-struct mh_option mh_option[] = {
-  { "clear",   MH_OPT_BOOL },
-  { "form",    MH_OPT_ARG, "formatfile" },
-  { "format",  MH_OPT_ARG, "string" },
-  { "header",  MH_OPT_BOOL },
-  { "width",   MH_OPT_ARG, "number" },
-  { "reverse", MH_OPT_BOOL },
-  { "file",    MH_OPT_ARG, "file" },
-  { NULL }
-};
+static char prog_doc[] = N_("Produce a one line per message scan listing");
+static char args_doc[] = N_("[MSGLIST]");
 
 static int clear;
-static char *format_str = mh_list_format;
 
-static int width = 80;
+static int width;
 static int reverse;
 static int header;
 
 static mh_format_t format;
-
+static mh_fvm_t fvm;
 static mu_msgset_t msgset;
 
-static int list_message (size_t num, mu_message_t msg, void *data);
-void print_header (mu_mailbox_t mbox);
-void clear_screen (void);
+static struct mu_option options[] = {
+  { "clear",   0, NULL, MU_OPTION_DEFAULT,
+    N_("clear screen after displaying the list"),
+    mu_c_bool, &clear },
+  { "form",    0, N_("FILE"),   MU_OPTION_DEFAULT,
+    N_("read format from given file"),
+    mu_c_string, &format, mh_opt_parse_formfile },
+  { "format",  0, N_("FORMAT"), MU_OPTION_DEFAULT,
+    N_("use this format string"),
+    mu_c_string, &format, mh_opt_parse_format },
+  { "header",  0, NULL,   MU_OPTION_DEFAULT,
+    N_("display header"),
+    mu_c_bool, &header },
+  { "width",   0, N_("NUMBER"), MU_OPTION_DEFAULT,
+    N_("set output width"),
+    mu_c_int, &width },
+  { "reverse",  0, NULL,   MU_OPTION_DEFAULT,
+    N_("list messages in reverse order"),
+    mu_c_bool, &reverse },
+  { "file",     0, N_("FILE"),   MU_OPTION_HIDDEN,
+    N_("[not yet implemented]"),
+    mu_c_string, NULL, mh_opt_notimpl },
+  MU_OPTION_END
+};
 
-static error_t
-opt_handler (int key, char *arg, struct argp_state *state)
+static void print_header (mu_mailbox_t mbox);
+static void clear_screen (void);
+
+static int
+list_message (size_t num MU_ARG_UNUSED, mu_message_t msg,
+	      void *data MU_ARG_UNUSED)
 {
-  switch (key)
-    {
-    case ARG_FOLDER: 
-      mh_set_current_folder (arg);
-      break;
-      
-    case ARG_CLEAR:
-      clear = is_true(arg);
-      break;
-
-    case ARG_NOCLEAR:
-      clear = 0;
-      break;
-      
-    case ARG_FORM:
-      if (mh_read_formfile (arg, &format_str))
-	exit (1);
-      break;
-      
-    case ARG_FORMAT:
-      format_str = arg;
-      break;
-      
-    case ARG_HEADER:
-      header = is_true(arg);
-      break;
-      
-    case ARG_WIDTH:
-      width = strtoul (arg, NULL, 0);
-      if (!width)
-	{
-	  argp_error (state, _("invalid width"));
-	  exit (1);
-	}
-      break;
-      
-    case ARG_REVERSE:
-      reverse = is_true(arg);
-      break;
-
-    case ARG_NOREVERSE:
-      reverse = 0;
-      break;
-      
-    case ARG_FILE:
-      mh_opt_notimpl ("-file");
-      break;
-      
-    default:
-      return ARGP_ERR_UNKNOWN;
-    }
+  mh_fvm_run (fvm, msg);
   return 0;
 }
 
@@ -145,15 +82,13 @@ action (mu_observer_t o, size_t type, void *data, void *action_data)
   static int counter;
   mu_mailbox_t mbox;
   mu_message_t msg = NULL;
-  size_t num;
 
   if (type == MU_EVT_MESSAGE_ADD)
     {
       mbox = mu_observer_get_owner (o);
       counter++;
       mu_mailbox_get_message (mbox, counter, &msg);
-      mh_message_number (msg, &num);
-      list_message (num, msg, NULL);
+      mh_fvm_run (fvm, msg);
     }
   return 0;
 }
@@ -161,28 +96,22 @@ action (mu_observer_t o, size_t type, void *data, void *action_data)
 int
 main (int argc, char **argv)
 {
-  int index;
   mu_mailbox_t mbox;
   int status;
   size_t total = 0;
   
-  /* Native Language Support */
-  MU_APP_INIT_NLS ();
-
-  mh_argp_init ();
-  mh_argp_parse (&argc, &argv, 0, options, mh_option, args_doc, doc,
-		 opt_handler, NULL, &index);
-
-  if (mh_format_parse (format_str, &format))
-    {
-      mu_error (_("Bad format string"));
-      exit (1);
-    }
+  mh_getopt (&argc, &argv, options, MH_GETOPT_DEFAULT_FOLDER,
+	     args_doc, prog_doc, NULL);
+  if (!format)
+    format = mh_scan_format ();
 
+  mh_fvm_create (&fvm, MH_FMT_FORCENL);
+  mh_fvm_set_format (fvm, format);
+  mh_fvm_set_width (fvm, width ? width : mh_width ());
+  mh_format_destroy (&format);
+  
   mbox = mh_open_folder (mh_current_folder (), MU_STREAM_READ);
 
-  argc -= index;
-  argv += index;
   if ((argc == 0 || strcmp (argv[0], "all") == 0) && !reverse)
     {
       /* Fast approach */
@@ -217,13 +146,16 @@ main (int argc, char **argv)
     }
 
   clear_screen ();
+  mh_fvm_destroy (&fvm);
+
   mh_global_save_state ();
   mu_mailbox_close (mbox);
   mu_mailbox_destroy (&mbox);
+
   return status;
 }
 
-void
+static void
 print_header (mu_mailbox_t mbox)
 {
   if (header)
@@ -248,8 +180,8 @@ putstdout(char c)
 }
 #endif
 
-void
-clear_screen ()
+static void
+clear_screen (void)
 {
   if (clear)
     {
@@ -279,18 +211,3 @@ clear_screen ()
       fprintf (stdout, "\f");
     }
 }
-
-static int
-list_message (size_t num, mu_message_t msg, void *data)
-{
-  char *buffer;
-  int len;
-  
-  mh_format (&format, msg, num, width, &buffer);
-  printf ("%s", buffer);
-  len = strlen (buffer);
-  if (len > 0 && buffer[len-1] != '\n')
-    printf("\n");
-  free (buffer);
-  return 0;
-}
