@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2009-2012, 2014-2015 Free Software Foundation, Inc.
+   Copyright (C) 2009-2012, 2014-2017 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -36,7 +36,7 @@ api_nls_set_locale (PyObject *self, PyObject *args)
     return NULL;
 
   lset = mu_set_locale (locale);
-  return _ro (PyString_FromString (lset ? lset : ""));
+  return _ro (PyString_FromString (mu_prstr (lset)));
 }
 
 static PyObject *
